import {
  startOrPauseImprovementWork,
  changePatientRiskLevel,
  finalizeImprovementWork,
  createComment,
  createHospital,
  createImprovementWork,
  upvoteSuggestion,
  upvoteComment,
  createSuggestion,
  createTaskOnImprovementWork,
  createUnit,
  createUser,
  getObject,
  getImprovementWorks,
  getSuggestions,
  getUsers,
  getTasks,
  getUnits,
  getComments,
  getImprovementWorksForUser,
  getUsersForImprovementWork,
  addDocumentToImprovementWork,
  addMemberToImprovementWork,
  updateInstance,
  addMemberToTask,
  deleteInstance,
  auth,
  deleteAllDocumentsInCollection,
  getObjectById,
  db,
} from "./database.js";
import { createUserWithEmailAndPassword } from "firebase/auth";
import * as fs from "fs";
import * as readline from "readline";

//Reads all lines from a file and returns them in an arraylist
async function readAllLinesFromFile(filename: string): Promise<string[]> {
  try {
    const fileStream = fs.createReadStream(
      // Removed
        "../c3/client/Database/" + filename,
      // To get the correct file path to the mock data files
      //filename,
      "utf8"
    );
    const lines: string[] = [];

    
    const lineReader = readline.createInterface({
      input: fileStream,
      crlfDelay: Infinity,
    });
    for await (const line of lineReader) {
      lines.push(line);
    }
    
    return lines;
  } catch (error) {
    console.log("Failed to read the file" + filename);
    console.log(error);
    return []; // Return an empty array on error
  }
}
function toDate(date: string): Date {
  var dateArray = date.split(":");
  return new Date(
    parseInt(dateArray[0]),
    parseInt(dateArray[1]),
    parseInt(dateArray[2])
  );
}

//Splits line into an array based on the "splitOn" input
function splitLineOn(splitOn: string, line: string): string[] {
  return line.split(splitOn);
}

//Reads mockdata from the mockdatafile for hospitals and adds it to the database
async function addHospitalsToDatabase() {
  console.log("adding hospital");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/hospitals.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 2) {
      console.log(
        "There was a problem with line",
        lineCounter,
        "in hospitals.txt"
      );
    }
    await createHospital(parameterArray[0], parameterArray[1]);
  }
}
//Reads mockdata from the mockdatafile for unit and adds it to the database
async function addUnitToDatabase() {
  console.log("adding units");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/hospitalunits.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 3) {
      console.log(
        "There was a problem with line",
        lineCounter,
        "in hospitalsunits.txt"
      );
    }
    await createUnit(parameterArray[0], parameterArray[1], parameterArray[2]);
  }
}
//Reads mockdata from the mockdatafile for tasks and adds it to the database
async function addTaskToDatabase() {
  console.log("adding task");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/tasks.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 7) {
      console.log("There was a problem with line", lineCounter, "in tasks.txt");
    }
    await createTaskOnImprovementWork(
      parameterArray[0],
      parameterArray[1],
      parameterArray[2],
      toDate(parameterArray[3]),
      toDate(parameterArray[4]),
      parameterArray[5],
      parameterArray[6]
    );
  }
}

//Reads mockdata from the mockdatafile for comment and adds it to the database
async function addCommentToDatabase() {
  console.log("adding comments");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/comments.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 4) {
      console.log(
        "There was a problem with line",
        lineCounter,
        "in comments.txt"
      );
    }
    await createComment(
      parameterArray[3],
      parameterArray[1],
      parameterArray[2]
    );
  }
}

//Reads mockdata from the mockdatafile for improvementWork and adds it to the database
async function addImprovementWorkToDatabase() {
  console.log("adding improvementWorks");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/improvementWorks.txt");
  const allUsers = await getObject("users");
  console.log(allUsers[0])
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    const members = getRandomUsers(10, allUsers);
    let membersid: string[]=[];
    console.log(members)
    for (const member of members){
      console.log("i for each loop")
      console.log(member)
        membersid.push(member.id);
    };
    console.log(membersid)
    console.log("efter for each loop")

    //All created improvementWorks starts as ongoing
    await createImprovementWork(
      parameterArray[0], // id
      parameterArray[1], // name
      parameterArray[2], // currentStage
      parameterArray[3], // content
      parameterArray[4], // unitId
      parameterArray[5], // purpose
      parameterArray[6], // patientRiskLevel
      parameterArray[7], // WRONG in mockdata, parameter is area
      members[0].id, // projectLeader
      membersid, // members
      parameterArray[8], // 
      parameterArray[9], //
      parameterArray[10], //
      parameterArray[11], //
      parameterArray[12], //
      parameterArray[13], //
      parameterArray[14], //
      parameterArray[15], //
      parameterArray[16], //
      parameterArray[17], //
      parameterArray[18], //
      parameterArray[19], //
      parameterArray[20], //
      parameterArray[21], //
      parameterArray[22], //
      parameterArray[23], //
      parameterArray[24], //
      parameterArray[25], //
      parameterArray[26], // 
    );
  }
}

//Reads mockdata from the mockdatafile for suggestion and adds it to the database
async function addSuggestionToDatabase() {
  console.log("adding suggestions");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/suggestions.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);

    var tmp = await getObjectById("users", parameterArray[1]);
    //All suggestions are set to public
    //All suggestions are needs
    await createSuggestion(
      parameterArray[0],
      parameterArray[1],
      parameterArray[2],
      parameterArray[3],
      parameterArray[4],
      parameterArray[5],
      parameterArray[6],
      parameterArray[7],
      parameterArray[8],
      parameterArray[9],
      true
    );
  }
}

//Reads mockdata from the mockdatafile for users and adds it to the database
async function addUsersToDatabase() {
  console.log("adding users");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/users.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 7) {
      console.log("There was a problem with line", lineCounter, "in users.txt");
    }
    console.log(parameterArray[4])
    let uid  = "";
    await createUserWithEmailAndPassword(auth, parameterArray[4], parameterArray[5])
    .then( async (userCredential) => {
      uid = userCredential.user.uid;
    })
    .catch((error) => {
      const errorMessage = error.message;
      console.log(errorMessage)
    })
    await createUser(
      parameterArray[0],
      parameterArray[1],
      parameterArray[2],
      parameterArray[3],
      parameterArray[4],
      parameterArray[6].includes("Projektledare"), //parameterArray[6] == 'True' if manager
      parameterArray[6],
      uid
    );
  }
}

// Returns 'num' random users from all users in the database
function getRandomUsers(num: number, allUsers: any[]) {
  // const allUsers = await getUsers({});
  const selectedUsers: any[] = [];

  for (let i = 0; i < num; i++) {
    let randomIndex = Math.floor(Math.random() * allUsers.length);
    selectedUsers.push(allUsers[randomIndex]);
  }
  return selectedUsers;
}

// TODO: Ev. uppatera denna funktion!
//Adds users to tasks in improvementWorks they are active on based on the mockdata
//denna är inte helt klar heller
// This function has a triple nested for loop, so it is very slow.
async function addMockmembersToTasks() {
  console.log("adding members to task");
  var improvementWorks = await getObject("improvementWork");
  var isResponsible: boolean;
  for (const improvementWork of improvementWorks) {
    var tasksInImprovementWork = await getTasks({improvementWorkId: improvementWork.id});
    for (const task of tasksInImprovementWork) {
      isResponsible = true;
      const improvementWorkMembers = await getUsersForImprovementWork(improvementWork.id)
      var members = getRandomUsers(5, improvementWorkMembers);
      for (const member of members) {
        await addMemberToTask(
          member.id,
          task.id,
          isResponsible
        );
        isResponsible = false;
      }
      
    }
  }
}

async function resetData() {
  try {
    await deleteData();
    await addData();
  } catch (error) {
    console.log(error);
  }
}

async function deleteData() {
  try {
    await deleteAllDocumentsInCollection("hospital");
    await deleteAllDocumentsInCollection("improvementWork");
    await deleteAllDocumentsInCollection("suggestion");
    await deleteAllDocumentsInCollection("task");
    await deleteAllDocumentsInCollection("unit");
    await deleteAllDocumentsInCollection("users");
    await deleteAllDocumentsInCollection("userImprovementWork");
    await deleteAllDocumentsInCollection("user");
    await deleteAllDocumentsInCollection("comment");
    await deleteAllDocumentsInCollection("document");
    await deleteAllDocumentsInCollection("userImprovementWork");
    await deleteAllDocumentsInCollection("userTask");
    console.log("Done deleting data");
  } catch (error) {
    console.log(error);
  }
}

async function addData() {
  try {
    await addHospitalsToDatabase();
    await addUnitToDatabase();
    await addUsersToDatabase();
    await addImprovementWorkToDatabase();
    await addTaskToDatabase();
    await addSuggestionToDatabase();
    await addCommentToDatabase();
    await addMockmembersToTasks();
    console.log("Done adding data");
  } catch (error) {
    console.log(error);
  }
}

async function testTasks() {
  try {
    let tasks = await getTasks({
      improvementWorkId: "proj_002"
    });

    if (tasks.length == 0) {
      console.log("No such tasks found");
    }

    tasks.forEach((task, key) => {
      // console.log("Task number " + key + ": ");
      console.log(task);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testUsers() {
  try {
    // REMOVE
    // let users = await getUsers("unit_234vwx");
    let users = await getUsers({ 
      // unitId: "unit_234vwx",
      hsaId: "micbro012" 
    });
    if (users.length == 0) {
      console.log("No such users found");
    }
    users.forEach((user, key) => {
      console.log("User number " + key + ": ");
      console.log(user);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testSuggestions() {
  try {
    // REMOVE
    // let suggestions = await getSuggestions("micbro012", "unit_jkl012", "dateCreated");
    let suggestions = await getSuggestions({
      // creatorHsaId: "johsmi123",
      unitId: "unit_abc123",
      // sortBy: "dateCreated",
      // id: "suggestion1"
    });
    if (suggestions.length == 0) {
      console.log("No such suggestions found");
    }
    suggestions.forEach((suggestion, key) => {
      console.log("Suggestion number " + key + ": ");
      console.log(suggestion);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testImprovementWorks() {
  try {
    // REMOVE
    // let improvementWorks = await getImprovementWorks(undefined, undefined, false, true, undefined);
    let improvementWorks = await getImprovementWorks({
      // ongoing: false,
      // nolanStage: "Plan",
      // id: "proj_001"
      patientRiskLevel: "Orange"
    });
    
    if (improvementWorks.length == 0) {
      console.log("No such improvement works found");
    }
    improvementWorks.forEach((improvementWork, key) => {
      console.log("Improvement work number " + key + ": ");
      console.log(improvementWork);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testUnits() {
  try {
    let units = await getUnits({ 
      // hospitalId: "elasmo"
      unitId: "unit_012jkl"
    });
    units.forEach((unit, key) => {
      console.log("Unit number " + key + ": ");
      console.log(unit);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testComments() {
  try {
    let comments = await getComments({ suggestionId: "suggestion1" });
    if (comments.length == 0) {
      console.log("No such comments found");
    }
    comments.forEach((comment, key) => {
      console.log("Comment number " + key + ": ");
      console.log(comment);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testFinalizeImprovementWork() {
  try {
    await finalizeImprovementWork("alijoh234", "proj_003");
  } catch (error) {
    console.log(error);
  }
}

async function testImprovementWorksForUser(userHsaId: string) {
  try {
    let improvementWorks = await getImprovementWorksForUser(userHsaId);
    if (improvementWorks.length == 0) {
      console.log("No such improvement works found");
    }
    improvementWorks.forEach((improvementWork, key) => {
      console.log("Improvement work number " + key + ": ");
      console.log(improvementWork);
    });
  } catch (error) {
    console.log(error);
  }
}

async function testUsersForImprovementWork(improvementWorkId: string) {
  try {
    let users = await getUsersForImprovementWork(improvementWorkId);
    if (users.length == 0) {
      console.log("No such users found");
    }
    users.forEach((user, key) => {
      console.log("User with id " + user.id + ": ");
      console.log(user);
    });
  } catch (error) {
    console.log(error);
  }
}

async function updateLowerCaseNames(): Promise<void> {
  try {
    const improvementWorks = await getImprovementWorks({});
    
    improvementWorks.forEach(async (work) => {
      const lowercaseName = work.name.toLowerCase();
      const updates = { lowerCaseName: lowercaseName };
      
      await updateInstance('improvementWork', work.id, updates);
    });

    console.log('Lowercase names updated successfully.');
  } catch (error) {
    console.error('Error updating lowercase names:', error);
    throw error;
  }
}

async function updateUserRoles() {
  console.log("adding users");
  var lineCounter = 0;
  const lines = await readAllLinesFromFile("mockdata/users.txt");
  for (const line of lines) {
    lineCounter = lineCounter + 1;
    var parameterArray = splitLineOn(", ", line);
    if (parameterArray.length != 7) {
      console.log("There was a problem with line", lineCounter, "in users.txt");
    }
    console.log(parameterArray[4]) 
    await updateInstance("users", parameterArray[0], {role: parameterArray[6]});
  }
}

// testImprovementWorksForUser("micbro012");
// testUsersForImprovementWork("proj_002");

resetData();
// deleteData();
// addData();
// addMockmembersToTasks();
//deleteAllDocumentsInCollection("improvementWork");
//addImprovementWorkToDatabase();
//updateLowerCaseNames();
//testFinalizeImprovementWork();
//startOrPauseImprovementWork("micbro012", "proj_004")
// updateUserRoles();
// testImprovementWorks();
// changePatientRiskLevel("alijoh234", "proj_001", "Orange");
// testImprovementWorks();
// testSuggestions();
// testTasks();
// testUsers();
// testUnits();

// testComments();

// PASTE INTO BROWSER CONSOLE IN FIREBASE AUTHENTICATION PAGE TO REMOVE USERS
// setInterval(() => {
//   document.getElementsByClassName('edit-account-button mat-focus-indicator mat-menu-trigger mat-icon-button mat-button-base')[0].click()
//   Array.from(document.getElementsByClassName('mat-focus-indicator mat-menu-item ng-star-inserted')).at(-1).click();
//   document.getElementsByClassName('confirm-button mat-focus-indicator mat-raised-button mat-button-base mat-warn')[0].click()
// }, 1000)