import { initializeApp } from "firebase/app";
import {
  getFirestore,
  collection,
  doc,
  setDoc,
  updateDoc,
  DocumentData,
  arrayUnion,
  CollectionReference,
  getDocs,
  getDoc,
  where,
  query,
  deleteDoc,
  orderBy,
  arrayRemove,
  limit,
  QuerySnapshot,
} from "firebase/firestore";
import { getAuth } from "firebase/auth";
import { diffieHellman } from "crypto";

// C3 solutions databasen - ska användas för P&S
const firebaseConfig = {
  apiKey: "AIzaSyB7n9YIDCmeyzeZzteFnXqOUBijzEO59ls",
  authDomain: "backup-277c3.firebaseapp.com",
  projectId: "backup-277c3",
  storageBucket: "backup-277c3.appspot.com",
  messagingSenderId: "296015398098",
  appId: "1:296015398098:web:71e1d1ec2b0c370e5a590b",
  measurementId: "G-E933CXWWCG",
};

//Backup databas - ska användas av R&D
// const firebaseConfig = {
//    apiKey: "AIzaSyAMfD8V0belL6GxbuoehL__6YTFZH4pyoc",
//    authDomain: "company3-backup.firebaseapp.com",
//    projectId: "company3-backup",
//    storageBucket: "company3-backup.appspot.com",
//    messagingSenderId: "820650717800",
//    appId: "1:820650717800:web:7557c7734ed59d3f7281dd"
// };

//Bens egna reserv #nödfall
//  const firebaseConfig = {
//   apiKey: "AIzaSyBIsZq_yOsJljp1woIxndJO1GkW6soh-g4",
//   authDomain: "company3-456bb.firebaseapp.com",
//   projectId: "company3-456bb",
//   storageBucket: "company3-456bb.appspot.com",
//   messagingSenderId: "310033686522",
//   appId: "1:310033686522:web:63198bb0db679c98ee4e43",
//   measurementId: "G-PFLDQY00DB"
// };

const app = initializeApp(firebaseConfig);
export const auth = getAuth();
export const db = getFirestore(app);
const userCollection = collection(db, "users");
const unitCollection = collection(db, "unit");
const improvementWorkCollection = collection(db, "improvementWork");
const taskCollection = collection(db, "task");
const hospitalCollection = collection(db, "hospital");
const suggestionCollection = collection(db, "suggestion");
const knowledgeDocumentCollection = collection(db, "knowledgeDocument");
const userImprovementWorkCollection = collection(db, "userImprovementWork");
const userTaskCollection = collection(db, "userTask");

interface userType {
  id: string;
  email: string;
  firstName: string;
  lastName: string;
  manager: boolean;
  password: string;
  unitId: string;
  improvementWorks: string[];
  tasks: string[];
}
////////////////////////////////////////Setters///////////////////////////////////////////////////////////////////
// Function to create user a new user in database
// takes in userId, unitId, firstname, lastname, email and password as string
// throws error if transaction failed
async function createUser(
  hsaId: string,
  unitId: string,
  firstName: string,
  lastName: string,
  email: string,
  manager: boolean,
  role: string,
  uid: string
): Promise<void> {
  //Passwords needs to be hashed, part of atuh0?
  try {
    const userRef = doc(userCollection, hsaId);
    await setDoc(userRef, {
      id: userRef.id,
      uid: uid,
      unitId: unitId,
      firstName: firstName,
      lastName: lastName,
      email: email,
      role: role,
      manager: manager,
      notes: "",
    });
  } catch (error) {
    console.log(error);
  }
}

//Adds a new improvementWork to the database
// takes in Id, name, currentStage in PDSA, content, unitId refers to as string and if completed as boolean
// throws error if transaction failed
interface improvementWork {
  id?: string;
  name: string;
  lowerCaseName?: string;
  currentStage: string;
  content: string;
  purpose: string;
  dateStarted: Date;
  unitId: string;
  completed: boolean;
  ongoing: boolean;
  patientRiskLevel: string;
  area: string;
  p1: string;
  p2: string;
  p3: string;
  p4: string;
  p5: string;
  p6: string;
  p7: string;
  g1: string;
  g2: string;
  g3: string;
  g4: string;
  g5: string;
  s1: string;
  s2: string;
  s3: string;
  a1: string;
  a2: string;
  a3: string;
  a4: string;
  resources: string;
  endDate: Date;
  hospitalId: string;
}

async function createImprovementWork(
  id: string | null,
  name: string,
  currentStage: string,
  content: string, // Är detta description bör detta förtydligas. JA
  unitId: string,
  purpose: string, //Bör tas bort motsvaras av p2- mål med projektet
  patientRiskLevel: string,
  area: string,
  projectLeader: string, //Adding projectleader & members to the project
  members: string[],
  resources: string = "Inte definerat ännu",
  hospitalId: string,
  //Questions in planning stage
  p1: string = "",
  p2: string = "",
  p3: string = "",
  p4: string = "",
  p5: string = "",
  p6: string = "",
  p7: string = "",
  //Questions in Doing (G) stage
  g1: string = "",
  g2: string = "",
  g3: string = "",
  g4: string = "",
  g5: string = "",
  //Questions in stydy stage
  s1: string = "",
  s2: string = "",
  s3: string = "",
  //Questions in acting stage
  a1: string = "",
  a2: string = "",
  a3: string = "",
  a4: string = ""
): Promise<void> {
  //ID has to be a string
  try {
    const improvementWorkData: improvementWork = {
      name: name,
      lowerCaseName: name.toLowerCase(),
      currentStage: currentStage,
      content: content,
      purpose: purpose,
      dateStarted: new Date(),
      unitId: unitId,
      completed: false,
      ongoing: false,
      patientRiskLevel: patientRiskLevel,
      area: area,
      p1: p1,
      p2: p2,
      p3: p3,
      p4: p4,
      p5: p5,
      p6: p6,
      p7: p7,
      g1: g1,
      g2: g2,
      g3: g3,
      g4: g4,
      g5: g5,
      s1: s1,
      s2: s2,
      s3: s3,
      a1: a1,
      a2: a2,
      a3: a3,
      a4: a4,
      resources: resources,
      endDate: new Date(0),
      hospitalId: hospitalId,
    };

    if (id) {
      const improvementWorkRef = doc(improvementWorkCollection, id);
      await setDoc(improvementWorkRef, {
        id: improvementWorkRef.id,
        ...improvementWorkData,
      });
    } else {
      const improvementWorkRef = doc(improvementWorkCollection);
      await setDoc(improvementWorkRef, improvementWorkData);
      id = improvementWorkRef.id;
    }
    //adding the projectleader and users to the project
    addMemberToImprovementWork(projectLeader, id, false, true);

    members.forEach((member) => {
      addMemberToImprovementWork(member, id!, false, false);
    });
  } catch (error) {
    console.log("Error creating improvementWork: ", error);
    throw error;
  }
}

async function addMemberToTask(
  userHsaId: string,
  taskId: string,
  isResponsibleFor: boolean
): Promise<void> {
  try {
    const relationRef = doc(userTaskCollection);

    await setDoc(relationRef, {
      id: relationRef.id,
      userHsaId: userHsaId,
      taskId: taskId,
      isResponsibleFor: isResponsibleFor,
    });
  } catch (error) {
    console.log("Error adding member to task: ", error);
    throw error;
  }
}

async function addMemberToImprovementWork(
  userHsaId: string,
  improvementWorkId: string,
  isOwner: boolean,
  isResponsibleFor: boolean
): Promise<void> {
  try {
    const relationRef = doc(userImprovementWorkCollection);

    await setDoc(relationRef, {
      id: relationRef.id,
      userHsaId: userHsaId,
      improvementWorkId: improvementWorkId,
      isOwner: isOwner,
      isResponsibleFor: isResponsibleFor,
    });
  } catch (error) {
    console.log("Error adding member to improvementWork: ", error);
    throw error;
  }
}

/**
 * async function deleteSuggestion(suggestionId: string): Promise<void> {
  try {
    const suggestionRef = doc(suggestionCollection, suggestionId);
    await deleteDoc(suggestionRef);
  } catch (error) {
    console.log("Error deleting suggestion: ", error);
    throw error;
  }
}
 */

async function removeMemberFromImprovementWork(
  userHsaId: string,
  improvementWorkId: string
): Promise<void> {
  try {
    let constraints: any[] = [];

    if (userHsaId && improvementWorkId) {
      constraints.push(where("improvementWorkId", "==", improvementWorkId));
      constraints.push(where("userHsaId", "==", userHsaId));

      const q = query(userImprovementWorkCollection, ...constraints);
      const querySnapshot = await getDocs(q);

      if (!querySnapshot.empty) {
        const docSnapshot = querySnapshot.docs[0];
        await deleteDoc(doc(db, "userImprovementWork", docSnapshot.id));
        console.log("userImprovementWork with id ", docSnapshot.id, " deleted");
      }
    } else {
      console.log("Both improvementWorkId and userHsaId need to be provided!");
    }
  } catch (error) {
    console.log("Error removing member from improvementWork: ", error);
    throw error;
  }
}

//create task to improvementWork
// takes in improvementWorkId, name, deadline date, start date, stage in PDSA and content as string
// throws error if transaction failed
interface Task {
  id?: string; // Optional because it can be auto-generated
  improvementWorkId: string;
  name: string;
  deadlineDate: Date;
  startDate: Date;
  stage: string;
  content: string;
  completed: boolean;
}
async function createTaskOnImprovementWork(
  taskId: string | null,
  improvementWorkId: string,
  name: string,
  deadlineDate: Date,
  startDate: Date,
  stage: string,
  content: string
): Promise<void> {
  try {
    const taskData: Task = {
      improvementWorkId,
      name,
      deadlineDate,
      startDate,
      stage,
      content,
      completed: false,
    };

    if (taskId) {
      const taskRef = doc(taskCollection, taskId);
      await setDoc(taskRef, { id: taskRef.id, ...taskData });
    } else {
      const taskRef = doc(taskCollection);
      await setDoc(taskRef, taskData);
    }
  } catch (error) {
    console.log("Error creating task: ", error);
    throw error;
  }
}

//add a document to a improvementWork
// takes in, document url, name and improvementWork id refering to as string
// throws error if transaction failed
async function createdDocumentInKnowledgeBank(
  documentUrl: string,
  name: string,
  projectId: string
): Promise<void> {
  try {
    const docRef = doc(knowledgeDocumentCollection);
    await setDoc(docRef, {
      id: docRef.id,
      documentUrl: documentUrl,
      name: name,
      projectId: projectId,
    });
  } catch (error) {
    console.log("Error adding document: ", error);
    throw error;
  }
}

//add a document to a improvementWork
// takes in, document url, name and improvementWork id refering to as string
// throws error if transaction failed
async function addDocumentToImprovementWork(
  documentUrl: string,
  name: string,
  improvementWorkId: string
): Promise<void> {
  try {
    const improvementWorkRef = doc(
      improvementWorkCollection,
      improvementWorkId
    );
    const docRef = doc(improvementWorkRef, "documents");
    await setDoc(docRef, {
      id: docRef.id,
      documentUrl: documentUrl,
      timestamp: new Date(),
      name: name,
    });
  } catch (error) {
    console.log("Error adding document: ", error);
    throw error;
  }
}

async function getDocumentsByImprovementWork(
  improvementWorkId: string
): Promise<DocumentData[]> {
  try {
    console.log("Getting documents for improvementwork: ", improvementWorkId);
    // Query the userImprovementWork collection for relations involving this user
    const q = query(
      knowledgeDocumentCollection,
      where("projectId", "==", improvementWorkId)
    );
    const querySnapshot = await getDocs(q);

    // Fetch each related improvementWork document
    const documents = [];
    for (const doc of querySnapshot.docs) {
      const documentId = doc.data().id;
      const documentDoc = await getObjectById("knowledgeDocument", documentId);
      documents.push(documentDoc);
    }
    return documents;
  } catch (error) {
    console.error("Error getting documents for improvmentwork: ", error);
    throw error;
  }
}

//create a hospital
// takes in ID, name as string
// throws error if transaction failed
interface Hospital {
  id?: string; // Optional because it can be auto-generated
  name: string;
}
async function createHospital(
  hospitalId: string | null, // should be auto?
  name: string
): Promise<void> {
  try {
    const hospitalData: Hospital = {
      name,
    };

    if (hospitalId) {
      const hospitalRef = doc(hospitalCollection, hospitalId);
      await setDoc(hospitalRef, { id: hospitalRef.id, ...hospitalData });
    } else {
      const hospitalRef = doc(hospitalCollection);
      await setDoc(hospitalRef, hospitalData);
    }
  } catch (error) {
    console.log("Error creating hospital: ", error);
    throw error;
  }
}

//create a unit
// takes in id, hospitals id and name as string
// throws error if transaction failed
interface Unit {
  id?: string; // Optional because it can be auto-generated
  hospitalId: string;
  name: string;
}
async function createUnit(
  unitId: string | null, //should be auto?
  hospitalId: string,
  name: string
): Promise<void> {
  try {
    const unitData: Unit = {
      hospitalId,
      name,
    };

    if (unitId) {
      const unitRef = doc(unitCollection, unitId);
      await setDoc(unitRef, { id: unitRef.id, ...unitData });
    } else {
      const unitRef = doc(unitCollection);
      await setDoc(unitRef, unitData);
    }
  } catch (error) {
    console.log("Error creating unit: ", error);
    throw error;
  }
}

//creates a suggestion
// takes in ID, the creators hsaId, name, content and the responding unitId as string
// throws error if transaction failed
interface Suggestion {
  id?: string; // Optional because it can be auto-generated
  creatorHsaId: string;
  name: string;
  area: string;
  content: string;
  purpose: string;
  measure: string;
  ideas: string;
  //need: boolean
  unitId: string;
  departmentId: string;
  public: boolean;
  upvotes: string[];
  comments: Comment[];
  dateCreated: Date;
}
async function createSuggestion(
  id: string | null,
  creatorHsaId: string,
  name: string,
  area: string,
  content: string,
  purpose: string,
  measure: string,
  ideas: string,
  //need: boolean,
  unitId: string,
  departmentId: string,
  isPublic: boolean
): Promise<void> {
  try {
    const suggestionData: Suggestion = {
      creatorHsaId,
      name,
      area,
      content,
      purpose,
      measure,
      ideas,
      //need,
      unitId,
      departmentId,
      public: isPublic,
      upvotes: [],
      comments: [],
      dateCreated: new Date(),
    };

    if (id) {
      const suggestionRef = doc(suggestionCollection, id);
      await setDoc(suggestionRef, { id: suggestionRef.id, ...suggestionData });
    } else {
      const suggestionRef = doc(suggestionCollection);
      await setDoc(suggestionRef, suggestionData);
    }
  } catch (error) {
    console.log("Error creating suggestion: ", error);
    throw error;
  }
}
async function deleteSuggestion(suggestionId: string): Promise<void> {
  try {
    const suggestionRef = doc(suggestionCollection, suggestionId);
    await deleteDoc(suggestionRef);
  } catch (error) {
    console.log("Error deleting suggestion: ", error);
    throw error;
  }
}

// Upvote a suggestion for a user
// takes in suggestion id, user HSAid
// returns error if transaction failed
async function upvoteSuggestion(
  suggestionId: string,
  userHsaId: string
): Promise<boolean> {
  try {
    const suggestionRef = doc(suggestionCollection, suggestionId);
    const suggestionDoc = await getDoc(suggestionRef);

    // if already liked, unlike
    if (
      suggestionDoc.exists() &&
      suggestionDoc.data().upvotes.includes(userHsaId)
    ) {
      await updateDoc(suggestionRef, {
        upvotes: arrayRemove(userHsaId),
      });
      return false;
    } else {
      // Atomically add a update to the upvote array field.
      await updateDoc(suggestionRef, {
        upvotes: arrayUnion(userHsaId),
      });
      return true;
    }
  } catch (error) {
    console.log("Error upvoting suggestion: ", error);
    throw error;
  }
}
//creates a comment
// takes in commenters id, suggestion id and content as string
// throws error if transaction failed
async function createComment( // change to subcollection
  commenterHsaId: string,
  suggestionId: string,
  content: string
): Promise<void> {
  try {
    const suggestionRef = doc(suggestionCollection, suggestionId);
    const commentCollection = await collection(suggestionRef, "comment");
    const commentRef = await doc(commentCollection);
    await setDoc(commentRef, {
      id: commentRef.id,
      commenterHsaId: commenterHsaId,
      content: content,
      dateCreated: new Date(),
      upvotes: [],
    });
  } catch (error) {
    console.log("Error creating comment: ", error);
    throw error;
  }
}
// Upvote a comment for a user
// takes in suggestionId, commentId and userId as strings
// return error if transaction failed
async function upvoteComment(
  suggestionId: string,
  commentId: string,
  userHsaId: string
): Promise<void> {
  try {
    const suggestionRef = doc(suggestionCollection, suggestionId);
    const commentRef = doc(suggestionRef, "comment", commentId);
    // Atomically add a update to the upvote array field.
    await updateDoc(commentRef, {
      upvotes: arrayUnion(userHsaId),
    });
  } catch (error) {
    console.log("Error upvoting comment: ", error);
    throw error;
  }
}

//add a note to a user
// takes in userHsaId and content as string
// throws error if transaction failed
async function addNoteToUser(
  userHsaId: string,
  content: string
): Promise<void> {
  try {
    const userRef = doc(userCollection, userHsaId);
    await updateDoc(userRef, {
      notes: content,
      dateCreated: new Date(),
    });
  } catch (error) {
    console.log("Error updating document: ", error);
    throw error;
  }
}

// export to setters to be able to use in other files
export {
  createComment,
  createHospital,
  createImprovementWork,
  createSuggestion,
  createTaskOnImprovementWork,
  createUnit,
  createUser,
  createdDocumentInKnowledgeBank,
  addNoteToUser,
  addDocumentToImprovementWork,
  addMemberToImprovementWork,
  addMemberToTask,
  upvoteSuggestion,
  upvoteComment,
  deleteSuggestion,
  improvementWork,
};

//////////////////////////////////////////////GETTERS/////////////////////////////////////////////////////////////////////

// Get any object
// input is the database type as a string.
// output is the whole collection of that type.
async function getObject(collectionPath: string): Promise<any[]> {
  try {
    const collectionRef: CollectionReference<DocumentData> = collection(
      db,
      collectionPath
    );
    const querySnapshot = await getDocs(collectionRef);
    const result: any[] = [];

    querySnapshot.forEach((doc) => {
      result.push(doc.data());
    });

    return result;
  } catch (error) {
    console.error("Error getting collection data", error);
    throw error;
  }
}

/**
 * Asynchronously retrieves a single document object from the specified collection path and document ID.
 * If the document does not exist, logs a warning and returns null.
 *
 * @param {string} collectionPath - The path to the collection in the database ("improvementWork", "hospital", "suggestion", "task", "unit", "users")
 * @param {string} documentId - The ID of the document to retrieve.
 * @returns {Promise<any|null>} A promise that resolves to the document object if found, or null if not.
 *
 * @throws Will throw an error if there is an issue during retrieval from the database.
 */
async function getObjectById(
  collectionPath: string,
  documentId: string
): Promise<any | null> {
  try {
    const docRef = doc(db, collectionPath, documentId);
    const docSnapshot = await getDoc(docRef);

    if (docSnapshot.exists()) {
      return { id: docSnapshot.id, ...docSnapshot.data() };
    } else {
      console.warn(`No such document! ${collectionPath}/${documentId}`);
      return null;
    }
  } catch (error) {
    console.error("Error getting object", error);
    throw error;
  }
}

/**
 * Asynchronously retrieves comments for a given suggestion.
 *
 * @param {Object} params - The parameters for fetching comments.
 * @param {string} params.suggestionId - The ID of the suggestion to fetch comments for.
 * @param {"dateCreated"|"upvotes"} [params.sortBy="dateCreated"] - The criterion to sort the comments by. Defaults to "dateCreated".
 * @param {boolean} [params.ascending=false] - Determines if the sorting should be in ascending order. Defaults to false (descending).
 * @returns {Promise<any[]>} A promise that resolves to an array of comment objects.
 *
 * @throws Will throw an error if the comments cannot be retrieved.
 */
async function getComments({
  suggestionId,
  sortBy = "dateCreated",
  ascending = false,
}: {
  suggestionId: string;
  sortBy?: "dateCreated" | "upvotes";
  ascending?: boolean;
}): Promise<any[]> {
  try {
    const suggestionRef = doc(db, "suggestionCollection", suggestionId);

    const commentsQuery = query(
      collection(suggestionRef, "comment"),
      orderBy(sortBy, ascending ? "asc" : "desc")
    );

    const commentsSnapshot = await getDocs(commentsQuery);

    const comments: any[] = [];
    commentsSnapshot.forEach((doc) => {
      comments.push(doc.data());
    });

    return comments;
  } catch (error) {
    console.error("Error getting comments for suggestion: ", error);
    throw error;
  }
}

//Filtering suggestions on unitId AND creatorHsaId
//Sort suggestions on dateCreated OR upvotes

/**
 * Asynchronously retrieves an array of suggestions from the database, with various filters and sorting options.
 *
 * @param {Object} params - The parameters for filtering and sorting suggestions.
 * @param {string} [params.id] - Optional ID to filter suggestions by a specific identifier.
 * @param {string} [params.creatorHsaId] - Optional HSA ID to filter suggestions by the creator's HSA ID.
 * @param {string} [params.unitId] - Optional unit ID to filter suggestions by a specific unit.
 * @param {"dateCreated"|"upvotes"} [params.sortBy="dateCreated"] - Criterion to sort the suggestions by. Defaults to "dateCreated".
 * @param {boolean} [params.ascending=false] - Flag indicating if sorting should be ascending. Defaults to false (descending order).
 * @param {boolean} [params.isPublic=true] - Flag indicating whether to filter for public suggestions. Defaults to true.
 * @returns {Promise<any[]>} A promise that resolves to an array of suggestion objects.
 *
 * @throws Will throw an error if there is an issue retrieving suggestions.
 */
async function getSuggestions({
  id,
  creatorHsaId,
  name,
  unitId,
  sortBy = "dateCreated", // Default sort by dateCreated
  ascending = false, // Sort descending by default
  isPublic = true,
  departmentId,
}: {
  id?: string;
  creatorHsaId?: string;
  name?: string;
  unitId?: string;
  sortBy?: "dateCreated" | "upvotes" | "name";
  ascending?: boolean;
  isPublic?: boolean;
  departmentId?: string;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "suggestion");
    let constraints: any[] = [];
    if (id) {
      constraints.push(where("id", "==", id));
    }

    if (unitId) {
      constraints.push(where("unitId", "==", unitId));
    }

    if (creatorHsaId) {
      constraints.push(where("creatorHsaId", "==", creatorHsaId));
    }

    if (name) {
      constraints.push(where("name", "==", name));
      sortBy = "name";
    }

    if (departmentId) {
      constraints.push(where("departmentId", "==", departmentId));
    }

    constraints.push(where("public", "==", isPublic));

    constraints.push(orderBy(sortBy, ascending ? "asc" : "desc"));

    const q = query(collectionRef, ...constraints);

    const querySnapshot = await getDocs(q);

    const suggestions: any[] = [];

    querySnapshot.forEach((doc) => {
      suggestions.push({ id: doc.id, ...doc.data() });
    });

    return suggestions;
  } catch (error) {
    console.log("Error retrieving suggestions by department: ", error);
    throw error;
  }
}

/**
 * Asynchronously searches for hospital documents in the database.
 * It can filter the results by the hospital's name, ID, or both.
 * If no parameters are provided, it retrieves all hospitals.
 *
 * @param {Object} params - An object containing the search parameters.
 * @param {string} [params.hospitalName] - Optional name of the hospital to filter the search.
 * @param {string} [params.hospitalId] - Optional unique identifier of the hospital to filter the search.
 * @returns {Promise<any[]>} A promise that resolves to an array of hospital objects that match the search criteria.
 *
 * @throws Will throw an error if there is an issue during the retrieval process.
 */
async function getHospitals({
  hospitalName,
  hospitalId,
}: {
  hospitalName?: string;
  hospitalId?: string;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "hospital");

    let constraints: any[] = [];

    if (hospitalName) {
      constraints.push(where("name", "==", hospitalName));
    }

    if (hospitalId) {
      constraints.push(where("id", "==", hospitalId));
    }

    const q = query(collectionRef, ...constraints);

    const querySnapshot = await getDocs(q);

    const hospitals: any[] = [];

    querySnapshot.forEach((doc) => {
      hospitals.push({ id: doc.id, ...doc.data() });
    });

    return hospitals;
  } catch (error) {
    console.log("Error retrieving Hospitals", error);
    throw error;
  }
}

/**
 * Asynchronously retrieves an array of improvement work documents based on various filter criteria.
 * It supports filtering by ID, Nolan stage, unit ID, completion status, ongoing status, and patient risk level.
 * It also supports sorting by the start date of the improvement work in either ascending or descending order.
 *
 * @param {Object} params - An object containing the filter and sorting parameters.
 * @param {string} [params.id] - Optional identifier to filter the improvement works.
 * @param {string} [params.nolanStage] - Optional Nolan stage to filter the improvement works.
 * @param {string} [params.unitId] - Optional unit ID to filter the improvement works.
 * @param {boolean} [params.completed] - Optional flag to filter the improvement works by their completion status.
 * @param {boolean} [params.ongoing] - Optional flag to filter the improvement works by their ongoing status.
 * @param {string} [params.patientRiskLevel] - Optional patient risk level to filter the improvement works.
 * @param {boolean} [params.ascendingDates=false] - Optional flag for sorting by date in ascending order. Defaults to false.
 * @returns {Promise<any[]>} A promise that resolves to an array of improvement work objects that match the criteria.
 *
 * @throws Will throw an error if there is a problem retrieving the improvement works.
 */
async function getImprovementWorks({
  id,
  name,
  nolanStage,
  unitId,
  completed,
  ongoing,
  patientRiskLevel,
  ascendingDates = false, //Descending dates by default
}: {
  id?: string;
  name?: string;
  nolanStage?: string;
  unitId?: string;
  completed?: boolean;
  ongoing?: boolean;
  patientRiskLevel?: string;
  ascendingDates?: boolean;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "improvementWork");

    let constraints: any[] = [];

    if (id) {
      constraints.push(where("id", "==", id));
    }

    if (unitId) {
      console.log("inne i ifsatsen i databasen");
      constraints.push(where("unitId", "==", unitId));
    }

    if (nolanStage) {
      constraints.push(where("currentStage", "==", nolanStage));
    }

    if (completed != undefined) {
      constraints.push(where("completed", "==", completed));
    }

    if (ongoing !== undefined) {
      constraints.push(where("ongoing", "==", ongoing));
    }
    if (patientRiskLevel) {
      constraints.push(where("patientRiskLevel", "==", patientRiskLevel));
    }

    if (name) {
      constraints.push(where("lowerCaseName", ">=", name.toLowerCase()));
      constraints.push(where("lowerCaseName", "<=", name.toLowerCase() + "~"));
    } else {
      constraints.push(orderBy("dateStarted", ascendingDates ? "asc" : "desc"));
    }

    const q = query(collectionRef, ...constraints);

    const querySnapshot = await getDocs(q);

    const improvementWorks: any[] = [];
    querySnapshot.forEach((doc) => {
      improvementWorks.push({ id: doc.id, ...doc.data() });
    });

    return improvementWorks;
  } catch (error) {
    console.log("Error retrieving improvementWorks: ", error);
    throw error;
  }
}

// Gets improvementWorks where each field matches the union of elements in the input array for each fo the fields
async function getFilteredImprovementWorks({
  patientRiskLevels,
  nolanStages,
  units,
  ascendingDates = false, // Descending dates by default
  completed,
  hospitalIds,
}: {
  patientRiskLevels?: string[];
  nolanStages?: string[];
  units?: string[];
  ascendingDates?: boolean;
  hospitalIds?: string[];
  completed?: boolean | null; // Use boolean or null to handle filtering by completion status
}): Promise<any[]> {
  try {
    console.log(
      "Getting improvement works with the following constraints",
      patientRiskLevels,
      nolanStages,
      units,
      ascendingDates,
      hospitalIds
    );

    const collectionRef = collection(db, "improvementWork");

    let constraints: any[] = [];

    if (patientRiskLevels && patientRiskLevels.length) {
      constraints.push(where("patientRiskLevel", "in", patientRiskLevels));
    }

    if (nolanStages && nolanStages.length) {
      constraints.push(where("currentStage", "in", nolanStages));
    }

    if (units && units.length) {
      constraints.push(where("unitId", "in", units));
    }

    // If none of the array filters are provided, default to ordering by date.
    if (!patientRiskLevels!.length && !nolanStages!.length && !units!.length) {
      constraints.push(orderBy("dateStarted", ascendingDates ? "asc" : "desc"));
    }

    // Handle filtering by completion status
    if (completed !== null && completed !== undefined) {
      constraints.push(where("completed", "==", completed));
    }

    if (hospitalIds && hospitalIds.length) {
      constraints.push(where("hospitalId", "in", hospitalIds));
    }
    const q = query(collectionRef, ...constraints);

    const querySnapshot = await getDocs(q);

    const improvementWorks: any[] = [];
    querySnapshot.forEach((doc) => {
      improvementWorks.push({ id: doc.id, ...doc.data() });
    });

    return improvementWorks;
  } catch (error) {
    console.error("Error retrieving filtered improvementWorks: ", error);
    throw error;
  }
}

async function getImprovementWorkOwner(improvementWorkId: string) {
  try {
    let constraints: any[] = [];
    constraints.push(where("improvementWorkId", "==", improvementWorkId));
    constraints.push(where("isOwner", "==", true));

    const q = query(userImprovementWorkCollection, ...constraints);
    const querySnapshot = await getDocs(q);

    if (querySnapshot.empty) {
      console.log("Improvement work ", improvementWorkId, " has no owner");
      return null;
    } else {
      const owner = querySnapshot.docs[0].data().userHsaId;
      return owner;
    }
  } catch (error) {
    console.log(error);
  }
}

async function getImprovementWorkResponsible(improvementWorkId: string) {
  try {
    let constraints: any[] = [];
    constraints.push(where("improvementWorkId", "==", improvementWorkId));
    constraints.push(where("isResponsibleFor", "==", true));

    const q = query(userImprovementWorkCollection, ...constraints);
    const querySnapshot = await getDocs(q);

    if (querySnapshot.empty) {
      console.log(
        "Improvement work ",
        improvementWorkId,
        " has no responsible"
      );
      return null;
    } else {
      const responsibleId = querySnapshot.docs[0].data().userHsaId;
      const responsible = await getObjectById("users", responsibleId);
      return responsible;
    }
  } catch (error) {
    console.log(error);
  }
}

// TODO getUsersForTask!
/**
 * Asynchronously retrieves user documents from the database based on various filter criteria.
 * It can filter by the Health Savings Account ID (hsaId), unit ID, and unique identifier (uid).
 * Additionally, it can sort the results by first name in either ascending or descending order.
 *
 * @param {Object} params - An object containing the filter and sorting parameters.
 * @param {string} [params.hsaId] - Optional HSA ID to filter the users.
 * @param {string} [params.unitId] - Optional unit ID to filter the users.
 * @param {boolean} [params.ascendingFirstNames=false] - Optional flag to sort by first names in ascending order. Defaults to false (descending order).
 * @param {string} [params.uid] - Optional unique identifier to filter the users.
 * @returns {Promise<any[]>} A promise that resolves to an array of user objects that match the search criteria.
 *
 * @throws Will throw an error if there is an issue during the retrieval process.
 */
async function getUsers({
  hsaId,
  name,
  unitId,
  ascendingFirstNames = false, // Descending names by default
  uid,
}: {
  hsaId?: string;
  name?: string;
  unitId?: string;
  ascendingFirstNames?: boolean;
  uid?: string;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "users");

    let constraints: any[] = [];

    if (hsaId) {
      constraints.push(where("id", "==", hsaId));
    }

    if (unitId) {
      constraints.push(where("unitId", "==", unitId));
    }

    if (uid) {
      constraints.push(where("uid", "==", uid));
    }
    if (name) {
      constraints.push(where("name", "==", name));
    }

    constraints.push(
      orderBy("firstName", ascendingFirstNames ? "asc" : "desc")
    );

    const q = query(collectionRef, ...constraints);
    const querySnapshot = await getDocs(q);

    const users: any[] = [];
    querySnapshot.forEach((doc) => {
      users.push({ id: doc.id, ...doc.data() });
    });

    return users;
  } catch (error) {
    console.log("Error retrieving users by department: ", error);
    throw error;
  }
}

// TODO getTasksForUser!
/**
 * Asynchronously retrieves an array of task documents from the database based on several filter and sorting criteria.
 * Filters include task ID, user ID and their responsibility status, and related improvement work ID.
 * Results can be sorted by date either in ascending or descending order.
 *
 * @param {Object} params - Parameters to filter and sort the task results.
 * @param {string} [params.taskId] - Optional task ID to filter the tasks.
 * @param {string} [params.userId] - Optional user ID to identify tasks related to a user.
 * @param {boolean} [params.isResponsibleFor] - Optional flag to filter tasks where the user is marked as responsible.
 * @param {string} [params.improvementWorkId] - Optional improvement work ID to filter tasks related to a specific improvement work.
 * @param {boolean} [params.ascendingDates=false] - Optional flag to sort tasks by date in ascending order; defaults to descending order.
 * @returns {Promise<any[]>} A promise that resolves to an array of task objects that match the search criteria.
 *
 * @throws Will throw an error if there is an issue during the retrieval process.
 */
async function getTasks({
  // userHsaId,
  // isResponsibleFor,
  improvementWorkId,
  stage,
  ascendingDates = false, // Descending dates by default
}: {
  // userHsaId?: string;
  // isResponsibleFor?: boolean;
  improvementWorkId?: string;
  stage?: string;
  ascendingDates?: boolean;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "task");

    let constraints: any[] = [];

    if (stage) {
      constraints.push(where("stage", "==", stage));
    }

    if (improvementWorkId != undefined) {
      constraints.push(where("improvementWorkId", "==", improvementWorkId));
      constraints.push(orderBy("startDate", ascendingDates ? "asc" : "desc"));
    }

    const q = query(collectionRef, ...constraints);
    const querySnapshot = await getDocs(q);

    let tasks: any[] = [];
    querySnapshot.forEach((doc) => {
      tasks.push({ id: doc.id, ...doc.data() });
    });

    return tasks;
  } catch (error) {
    console.log("Error retrieving task by department: ", error);
    throw error;
  }
}

/**
 * Asynchronously retrieves an array of unit documents from the database based on various filter criteria.
 * Filters can be applied based on unit ID, hospital ID, and unit name.
 * Additionally, results can be sorted by unit names in ascending or descending order.
 *
 * @param {Object} params - The parameters object containing filter and sorting options.
 * @param {string} [params.unitId] - Optional unit ID to filter the units.
 * @param {string} [params.hospitalId] - Optional hospital ID to filter units within a specific hospital.
 * @param {string} [params.name] - Optional name to filter units by a specific name.
 * @param {boolean} [params.ascendingUnitNames=false] - Optional flag to sort units by name in ascending order, defaults to descending.
 * @returns {Promise<any[]>} A promise that resolves to an array of unit objects that match the search criteria.
 *
 * @throws Will throw an error if there is an issue during the retrieval process.
 */
async function getUnits({
  unitId,
  hospitalId,
  name,
  ascendingUnitNames = false, // Sort descending names by default
}: {
  unitId?: string;
  hospitalId?: string;
  name?: string;
  ascendingUnitNames?: boolean;
}): Promise<any[]> {
  try {
    const collectionRef = collection(db, "unit");

    let constraints: any[] = [];

    if (unitId) {
      constraints.push(where("id", "==", unitId));
    }

    if (name) {
      constraints.push(where("name", "==", name));
    }

    if (hospitalId) {
      constraints.push(where("hospitalId", "==", hospitalId));
      constraints.push(orderBy("name", ascendingUnitNames ? "asc" : "desc"));
    }

    const q = query(collectionRef, ...constraints);
    const querySnapshot = await getDocs(q);

    let units: any[] = [];
    querySnapshot.forEach((doc) => {
      units.push({ id: doc.id, ...doc.data() });
    });

    return units;
  } catch (error) {
    console.log("Error retrieving units by hospitalId: ", error);
    throw error;
  }
}

async function getImprovementWorksForUser(
  userHsaId: string
): Promise<DocumentData[]> {
  try {
    console.log("Getting improvementworks for user: ", userHsaId);
    // Query the userImprovementWork collection for relations involving this user
    const q = query(
      userImprovementWorkCollection,
      where("userHsaId", "==", userHsaId)
    );
    const querySnapshot = await getDocs(q);

    // Fetch each related improvementWork document
    const improvementWorks = [];
    for (const doc of querySnapshot.docs) {
      const improvementWorkId = doc.data().improvementWorkId;
      const improvementWorkDoc = await getObjectById(
        "improvementWork",
        improvementWorkId
      );
      improvementWorks.push(improvementWorkDoc);
    }
    return improvementWorks;
  } catch (error) {
    console.error("Error getting improvement works for user: ", error);
    throw error;
  }
}

async function getUsersForImprovementWork(
  improvementWorkId: string
): Promise<DocumentData[]> {
  try {
    // Query the userImprovementWork collection for relations involving this improvementWork
    const q = query(
      userImprovementWorkCollection,
      where("improvementWorkId", "==", improvementWorkId)
    );
    const querySnapshot = await getDocs(q);

    // Fetch each related user document
    const users = [];
    for (const doc of querySnapshot.docs) {
      const userHsaId = doc.data().userHsaId;
      const userDoc = await getObjectById("users", userHsaId);
      users.push(userDoc);
    }

    return users;
  } catch (error) {
    console.error("Error getting users for improvementWork: ", error);
    throw error;
  }
}

// export getters to be able to use in other files
export {
  getObject,
  getObjectById,
  getSuggestions,
  getImprovementWorks,
  getUsers,
  getTasks,
  getUnits,
  getComments,
  getHospitals,
  getImprovementWorksForUser,
  getUsersForImprovementWork,
  getImprovementWorkOwner,
  getImprovementWorkResponsible,
  getFilteredImprovementWorks,
  getDocumentsByImprovementWork,
};

////////////////////////////////////DELETING/////////////////////////////////////////////

// delete an instance of a type
async function deleteInstance(docType: string, docId: string) {
  try {
    const docRef = doc(db, docType, docId);
    await deleteDoc(docRef);
    console.log(`${docType} deleted successfully.`);
  } catch (error) {
    console.error(`Error deleting ${docType}:`, error);
    throw error;
  }
}

// delete all instances of a type
async function deleteAllDocumentsInCollection(
  collectionName: string
): Promise<void> {
  try {
    const collectionRef = collection(db, collectionName);
    const snapshot = await getDocs(collectionRef);

    // Delete all documents in the collection
    const deletePromises = snapshot.docs.map(async (doc: { ref: any }) => {
      await deleteDoc(doc.ref);
    });
    await Promise.all(deletePromises);

    console.log(
      `All documents in the ${collectionName} collection deleted successfully.`
    );
  } catch (error) {
    console.error(`Error deleting all documents in ${collectionName}:`, error);
    throw error;
  }
}

////////////////////////////////////UPDATING/////////////////////////////////////////////

// update a field in an instance
async function updateInstance(docType: string, docId: string, updates: any) {
  try {
    const docRef = doc(db, docType, docId);
    await updateDoc(docRef, updates);
    console.log(`${docType} updated successfully.`);
  } catch (error) {
    console.error(`Error updating ${docType}:`, error);
    throw error;
  }
}

// changes the ongoing status of an improvement work. if ongoing it is paused, if if paused onging.
// user must be a manager
async function startOrPauseImprovementWork(
  userId: string,
  improvementWorkId: string
): Promise<boolean> {
  try {
    let user: userType | null = await getObjectById("users", userId);
    let improvementWork: improvementWork | null = await getObjectById(
      "improvementWork",
      improvementWorkId
    );

    if (!user || !user.manager) {
      throw new Error("User can't change improvementWork status");
    } else {
      if (improvementWork) {
        let update = { ongoing: !improvementWork.ongoing };
        await updateInstance("improvementWork", improvementWorkId, update);
        console.log(`Improvement work ${improvementWorkId} status updated.`);
        return true;
      } else {
        console.error(
          `Improvement work with ID ${improvementWorkId} not found.`
        );
        return false;
      }
    }
  } catch (error) {
    console.error("Error starting/pausing improvement work:", error);
    throw error;
  }
}

async function changePatientRiskLevel(
  userId: string,
  improvementWorkId: string,
  patientRiskLevel: string
) {
  try {
    let user: userType | null = await getObjectById("users", userId);
    let improvementWork: improvementWork | null = await getObjectById(
      "improvementWork",
      improvementWorkId
    );

    if (!user || !user.manager) {
      throw new Error("User can't change patientRiskLevel in improvement work");
    } else {
      if (improvementWork) {
        let update = { patientRiskLevel: patientRiskLevel };
        await updateInstance("improvementWork", improvementWorkId, update);
        console.log(`Improvement work ${improvementWorkId} status updated.`);
      } else {
        console.error(
          `Improvement work with ID ${improvementWorkId} not found.`
        );
      }
    }
  } catch (error) {
    console.error("Error starting/pausing improvement work:", error);
    throw error;
  }
}

// finalizes the improvement work. ongoing is set to false.
// user must be a manager
async function finalizeImprovementWork(
  userId: string,
  improvementWorkId: string
) {
  try {
    let user: userType | null = await getObjectById("users", userId);
    if (!user || !user.manager) {
      return false;
    } else {
      let update = { completed: true, ongoing: false, endDate: new Date() };
      await updateInstance("improvementWork", improvementWorkId, update);
      return true;
    }
  } catch (error) {
    return false;
  }
}

function getAddAndDeleteMembersLists(
  listA: string[],
  listB: string[]
): { delete: string[]; add: string[] } {
  const deleteList = listA.filter((element) => !listB.includes(element));
  const addList = listB.filter((element) => !listA.includes(element));

  return {
    delete: deleteList,
    add: addList,
  };
}

export async function updateImprovementWorkMembers(
  improvementWorkId: string,
  projectLeaderId: string,
  membersIdList: string[]
) {
  const currentProjectLeader = await getImprovementWorkResponsible(
    improvementWorkId
  );

  if (currentProjectLeader.id === projectLeaderId) {
    // Do nothing if the project leader is the same
  } else {
    // Remove the current project leader and add the new one
    await removeMemberFromImprovementWork(
      improvementWorkId,
      currentProjectLeader.id
    );
    await addMemberToImprovementWork(
      projectLeaderId,
      improvementWorkId,
      false,
      true
    );
    console.log("ledare");
  }

  const currentMembers = await getUsersForImprovementWork(improvementWorkId);
  const currentMembersId: string[] = [];

  currentMembers.forEach((member) => {
    currentMembersId.push(member.id);
  });

  const differenceList = getAddAndDeleteMembersLists(
    currentMembersId,
    membersIdList
  );

  console.log("delete " + differenceList.delete);
  console.log("add " + differenceList.add);

  const indexOfProjectLeader = differenceList.delete.indexOf(projectLeaderId);

  if (indexOfProjectLeader !== -1) {
    alert(
      "Du kan inte ta bort projektledaren från ditt förbättringsarbete. För att göra detta byt projektledare först. Alla andra ändringar kommer genomföras."
    );
    differenceList.delete.splice(indexOfProjectLeader, 1);
  }

  differenceList.delete.forEach((deleteId) => {
    removeMemberFromImprovementWork(deleteId, improvementWorkId);
  });
  differenceList.add.forEach((addId) => {
    addMemberToImprovementWork(addId, improvementWorkId, false, false);
  });
}

export {
  deleteInstance,
  updateInstance,
  finalizeImprovementWork,
  deleteAllDocumentsInCollection,
  startOrPauseImprovementWork,
  changePatientRiskLevel,
};
//////////////////////////////// for later sprint

//function add_upvotes

//function add_notes

//function add_intrested_users
