"use strict";
import {
  getImprovementWorkResponsible,
  getImprovementWorks,
  getObject,
  getObjectById,
  getFilteredImprovementWorks,
} from "../Database/database.js";
import {
  loadProjectPageP,
  loadProjectPageA,
  loadProjectPageG,
  loadProjectPageS,
} from "../SidebarAndHeader/pageHandler.js";
import {
  loadHospitals,
  loadUnits,
  clearContent,
  removeCardsFromDiv,
} from "../ideaBank/ideaBank.js";

import { formatDate } from "../projectPage/projectPage.js";

let projectCards: projectCard[] = [];
let filteredImprovementWorks: any[] = [];

export class projectCard {
  public name: string; // Projectowner
  public department: string;
  public title: string;
  public description: string;
  public id: number;
  public projectId: string;
  public stage: any;
  public dateStarted: any;
  public endDate: any;
  public completed: boolean;

  constructor(
    id: number,
    name: string,
    department: string,
    title: string,
    description: string,
    projectId: string,
    currentStage: string,
    dateStarted: any,
    endDate: any,
    completed: boolean
  ) {
    this.name = name;
    this.department = department;
    this.title = title;
    this.description = description;
    this.id = id;
    this.projectId = projectId;
    this.stage = currentStage;
    this.dateStarted = dateStarted;
    this.endDate = endDate;
    this.completed = completed;
  }

  showMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId);
    if (blockElement) {
      blockElement.style.display = "flex";
    }

    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "^ Stäng detaljer";
    }
  }

  closeMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId) as HTMLSelectElement;
    if (blockElement) {
      blockElement.style.display = "none";
    }
    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "> Mer information";
    }
  }

  loadInitiatePage() {
    alert("The initiate project page should now be loaded");
  }

  generateHTML(): string {
    // Set default styles for stageSymbol (P, D, S, A, Avslutat)
    let stageSymbol = "";
    let stageFontSize = "50px";
    let stageMarginLeft = "35px";
    let stageMarginTop = "10px";

    // Set projectDates to dateStarted
    let projectDates = formatDate(this.dateStarted);

    if (this.completed) {
      stageSymbol = "Avslutat";
      stageFontSize = "25px";
      stageMarginLeft = "5px";
      stageMarginTop = "30px";

      // Append endDate to projectDates
      projectDates += " till " + formatDate(this.endDate);
    } else if (this.stage == "Plan") {
      stageSymbol = "P";
    } else if (this.stage == "Do") {
      stageSymbol = "G";
    } else if (this.stage == "Study") {
      stageSymbol = "S";
    } else if (this.stage == "Act") {
      stageSymbol = "A";
    }

    return `
    <div class="totalExpandedOngoingWorkCard">
      <div style="cursor: default;" class="flex-container ongoingCard shadow" id="${this.projectId}">
        <div class="ongoingCardImageContainer toSinglePage">
          <div id="borderPinkFaded">
            <div class = "flex-box ongoingCardImage toSinglePage">
              <div id="currentStageLetter${this.projectId}" style=
              "font-size: ${stageFontSize}; 
              margin-left: ${stageMarginLeft}; 
              margin-top: ${stageMarginTop};
              font-weight: 
              bolder;
              color: #845380;
              position: inherit; 
              ">${stageSymbol}</div>
            </div>
          </div>
        </div>
        <div class="div ongoingCardInfo toSinglePage">
        <p class="toSinglePage" id="ongoingCardIdeaTitle">${this.title}</p>
        <p class="toSinglePage" id="ongoingCardDepartment">${this.department}</p>
          <p class="toSinglePage" id="ongoingCardManager">${this.name}</p>
          <p class="toSinglePage" id="ongoingCardDatesCurrentProjects${this.projectId}" style="font-size: 13px; font-style: italic; margin-top: 5px;">${projectDates} </p>
        </div>
        <button id="project-button-${this.projectId}" data-project-id="${this.id}" class="projectButton fs-6 text-white d-flex pb-2 pt-2" role="button" style="height: 10%; min-width: 15vw; white-space: nowrap; align-items: center; font-family:'Roboto', sans-serif;">Gå till Förbättringsarbetet</button>
        <div class="container ongoingSeeCardDetails toSinglePage">
          <div class="ongoingSeeCardDetailsInner">
            <div id="ongoingSeeDetailsButton${this.projectId}" class = "moreInfoClass">> Mer information</div>
          </div>
        </div>
      </div> 
      <div class = "expandedOngoingWorkCard" id="expandedOngoingWorkCard${this.projectId}" style="display: none;">
        <div class ="expandedTextOngoingWorkCard">
          ${this.description}
        </div>
      </div>
    </div> `;
  }
}

export async function loadPage(listenersLoaded: boolean) {
  const container = document.querySelector("#container-fluid");

  if (!container) return; // Stop if the container isn't found
  projectCards = [];

  // Initialize empty lists of checked hospital and unit ids
  let checkedHospitalIds: string[] = [];
  let checkedUnitIds: string[] = [];
  let checkedNolanStages: string[] = [];
  let checkedPatientRiskLevels: string[] = [];
  let checkedWorkStatus: boolean = false;
  let ascendingDates: boolean = false;

  fetch("currentProjects/currentProjects.html")
    .then((response) => response.text())
    .then((data) => {
      // Create a temporary container element to hold the loaded content
      const tempContainer = document.createElement("div");
      tempContainer.innerHTML = data;

      // Select the specific element with the class you want
      const specificElement = tempContainer.querySelector(".filterBardiv");

      const createHospitalCheckBoxContainer = tempContainer.querySelector(
        "#hospitalSelection"
      ) as HTMLSelectElement;
      const createUnitCheckBoxContainer = tempContainer.querySelector(
        "#unitSelection"
      ) as HTMLSelectElement;
      const createNolanStageCheckBoxContainer = tempContainer.querySelector(
        "#nolanStageSelection"
      ) as HTMLSelectElement;
      const createPatientRiskLevelCheckBoxContainer =
        tempContainer.querySelector(
          "#patientRiskLevelSelection"
        ) as HTMLSelectElement;
      const createWorkStatusCheckboxContainer = tempContainer.querySelector(
        "#projectStatusSelection"
      ) as HTMLSelectElement;
      const createOrderSelectionContainer = tempContainer.querySelector(
        "#orderSelection"
      ) as HTMLSelectElement;

      if (createHospitalCheckBoxContainer) {
        createHospitalCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes = createHospitalCheckBoxContainer.querySelectorAll(
              'input[type="checkbox"]'
            );
            console.log("createHospitalCheckBoxContainer");

            // Empty list of checked hospitals
            checkedHospitalIds = [];

            // Array to store IDs of checked checkboxes
            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedHospitalIds.push(inputCheckbox.id);
              }
            });

            // Log all checked hospital IDs
            console.log("Checked hospitals:", checkedHospitalIds);

            filteredImprovementWorks = await getFilteredImprovementWorks({
              patientRiskLevels: checkedPatientRiskLevels,
              nolanStages: checkedNolanStages,
              units: checkedUnitIds,
              completed: checkedWorkStatus,
              hospitalIds: checkedHospitalIds,
            });
            removeCardsFromDiv("div.projectCardCardDiv");
            getProjects(container, listenersLoaded, filteredImprovementWorks);

            // Future change: Only clear the unitIds for unchecked hospitals!
            clearContent("unitSelection");
            loadUnits(checkedHospitalIds);
          }
        );
      }

      if (createUnitCheckBoxContainer) {
        createUnitCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes = createUnitCheckBoxContainer.querySelectorAll(
              'input[type="checkbox"]'
            );

            console.log("createUnitCheckBoxContainer");
            // Empty list of checked units
            checkedUnitIds = [];

            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedUnitIds.push(inputCheckbox.id);
              }
            });

            filteredImprovementWorks = await getFilteredImprovementWorks({
              patientRiskLevels: checkedPatientRiskLevels,
              nolanStages: checkedNolanStages,
              units: checkedUnitIds,
              completed: checkedWorkStatus,
            });
            removeCardsFromDiv("div.projectCardCardDiv");
            getProjects(container, listenersLoaded, filteredImprovementWorks);
          }
        );
      }

      if (createNolanStageCheckBoxContainer) {
        createNolanStageCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes =
              createNolanStageCheckBoxContainer.querySelectorAll(
                'input[type="checkbox"]'
              );

            // Empty list of checked hospitals
            checkedNolanStages = [];

            // Array to store IDs of checked checkboxes
            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedNolanStages.push(inputCheckbox.id);
                checkedNolanStages.push(nolanStageMapping[inputCheckbox.id]);
                console.log("checkedNolanStages: ", checkedNolanStages);
              }
            });

            filteredImprovementWorks = await getFilteredImprovementWorks({
              patientRiskLevels: checkedPatientRiskLevels,
              nolanStages: checkedNolanStages,
              units: checkedUnitIds,
              completed: checkedWorkStatus,
            });
            removeCardsFromDiv("div.projectCardCardDiv");
            getProjects(container, listenersLoaded, filteredImprovementWorks);
          }
        );
      }

      if (createPatientRiskLevelCheckBoxContainer) {
        createPatientRiskLevelCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes =
              createPatientRiskLevelCheckBoxContainer.querySelectorAll(
                'input[type="checkbox"]'
              );
            console.log("createPatientRiskLevelCheckBoxContainer");

            // Empty list of checked hospitals
            checkedPatientRiskLevels = [];

            // Array to store IDs of checked checkboxes
            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedPatientRiskLevels.push(inputCheckbox.id);
                checkedPatientRiskLevels.push(
                  patientRiskLevelMapping[inputCheckbox.id]
                );
                console.log(
                  "checkedPatientRiskLevels: ",
                  checkedPatientRiskLevels
                );
              }
            });

            filteredImprovementWorks = await getFilteredImprovementWorks({
              patientRiskLevels: checkedPatientRiskLevels,
              nolanStages: checkedNolanStages,
              units: checkedUnitIds,
              completed: checkedWorkStatus,
            });
            removeCardsFromDiv("div.projectCardCardDiv");
            getProjects(container, listenersLoaded, filteredImprovementWorks);
          }
        );
      }

      if (createWorkStatusCheckboxContainer) {
        console.log(
          "Checkbox container found:",
          createWorkStatusCheckboxContainer
        );

        createWorkStatusCheckboxContainer.addEventListener(
          "change",
          async function (event) {
            console.log("Checkbox changed");
            const checkboxes =
              createWorkStatusCheckboxContainer.querySelectorAll(
                'input[type="checkbox"]'
              );

            // Use a boolean variable to store the completion status
            let checkedWorkStatus = false;

            // Check the checkboxes to determine the completion status
            checkboxes.forEach((checkbox) => {
              const inputCheckbox = checkbox as HTMLInputElement;
              if (inputCheckbox.checked) {
                checkedWorkStatus = true;
              }
            });

            console.log("Filtering by completion status:", checkedWorkStatus);

            filteredImprovementWorks = await getFilteredImprovementWorks({
              patientRiskLevels: checkedPatientRiskLevels,
              nolanStages: checkedNolanStages,
              units: checkedUnitIds,
              completed: checkedWorkStatus,
            });

            console.log(
              "Filtered improvementWorks after applying completion status filter:",
              filteredImprovementWorks
            );

            removeCardsFromDiv("div.projectCardCardDiv");
            getProjects(container, listenersLoaded, filteredImprovementWorks);
          }
        );
      } else {
        console.error("Checkbox container not found!");
      }

      if (createOrderSelectionContainer) {
        createOrderSelectionContainer.addEventListener(
          "change",
          async function (event) {
            const inputCheckbox = event.target as HTMLInputElement;

            ascendingDates = inputCheckbox.checked;

            removeCardsFromDiv("div.projectCardCardDiv");
            sortProjects(container, listenersLoaded, ascendingDates);
          }
        );
      }

      if (specificElement) {
        // Insert the specific element into the target element
        container.insertBefore(specificElement, container.firstChild);
      }
    })
    .catch((error) => {
      console.error("Failed to load the external HTML file:", error);
    });
  filteredImprovementWorks = await getImprovementWorks({});
  console.log("Filtered improvementWorks: ", filteredImprovementWorks);
  removeCardsFromDiv("div.projectCardCardDiv");
  getProjects(container, listenersLoaded, filteredImprovementWorks);

  const searchBar = document.getElementById(
    "searchBarImprovement"
  ) as HTMLInputElement;
  searchBar?.addEventListener("keydown", function (event) {
    if (event.key === "Enter") {
      performSearch(searchBar, container);
    }
  });
  console.log("Loading hospitals");
  clearContent("hospitalSelection");
  loadHospitals();
  loadNolanStages();
  loadPatientRiskLevels();
  loadWorkStatus();
}

export async function loadNolanStages() {
  const selectStageHTML = $("#nolanStageSelection"); // loads the select type from html
  const allStages = ["Planera", "Göra", "Studera", "Agera"];
  allStages.forEach((stage) => {
    const generatedElement = document.createRange().createContextualFragment(
      `
    <div class="form-check">
      <input class="form-check-input" type="checkbox" value="" id="` +
        stage +
        `">
      <label class="form-check-label" for="flexCheckDefault">` +
        stage +
        `</label>
    </div>`
    );
    selectStageHTML.append(generatedElement);
    console.log("Adding HTML for for stage ", stage);
  });
}

export async function loadPatientRiskLevels() {
  const selectPatientRiskLevelHTML = $("#patientRiskLevelSelection"); // loads the select type from html
  const allRiskLevels = ["Grön", "Blå", "Gul", "Röd"];
  allRiskLevels.forEach((riskLevel) => {
    const generatedElement = document.createRange().createContextualFragment(
      `
    <div class="form-check">
      <input class="form-check-input" type="checkbox" value="" id="` +
        riskLevel +
        `">
      <label class="form-check-label" for="flexCheckDefault">Risknivå ` +
        riskLevel +
        `</label>
    </div>`
    );
    selectPatientRiskLevelHTML.append(generatedElement);
    console.log("Adding HTML for for risk level ", riskLevel);
  });
}

async function loadWorkStatus() {
  const selectWorkStatusHTML = $("#projectStatusSelection");
  const allWorkStatus = ["Visa avslutade"];
  allWorkStatus.forEach((completed) => {
    const generatedElement = document.createRange().createContextualFragment(
      `
    <div class="form-check">
      <input class="form-check-input" type="checkbox" value="" id="` +
        completed +
        `">
      <label class="form-check-label" for="flexCheckDefault">` +
        completed +
        `</label>
    </div>`
    );
    selectWorkStatusHTML.append(generatedElement);
    console.log("Adding HTML for for completed ", completed);
  });
}

export function sortProjects(
  selectElement: Element,
  listenersLoaded: boolean,
  ascendingDates: boolean
) {
  // Sort in reversed order if ascendingDates is true
  const cardsToIterate = ascendingDates
    ? [...projectCards].reverse()
    : projectCards;

  // Loop through the saved array of cards and generate HTML for each card
  for (const card of cardsToIterate) {
    let cardHTML = card.generateHTML();
    let div = document.createElement("div");
    div.className = "projectCardCardDiv"; // Set the class name(s) here
    div.innerHTML = cardHTML;
    if (!selectElement) return;
    selectElement.appendChild(div); // Stop if the container isn't found
    addEventListener(card.projectId, card);
  }
}

export async function getProjects(
  selectElement: Element,
  listenersLoaded: boolean,
  improvementWorks: any[]
) {
  let id = 0;
  projectCards = [];
  improvementWorks.forEach(async (improvementWork) => {
    let title = improvementWork.name;
    let department = await getObjectById("unit", improvementWork.unitId);
    let projectLeader = await getImprovementWorkResponsible(improvementWork.id);
    if (department) {
      let card = new projectCard(
        id,
        projectLeader.firstName + " " + projectLeader.lastName,
        department.name,
        title,
        improvementWork.content,
        improvementWork.id,
        improvementWork.currentStage,
        improvementWork.dateStarted,
        improvementWork.endDate,
        improvementWork.completed
      );
      projectCards.push(card);
      let cardHTML = card.generateHTML();
      let div = document.createElement("div");
      div.className = "projectCardCardDiv"; // Set the class name(s) here
      div.innerHTML = cardHTML;
      if (!selectElement) return;
      selectElement.appendChild(div); // Stop if the container isn't found
      addEventListener(improvementWork.id, card);

      id++;
    } else {
      console.log(improvementWork.unitId + " is not a unit"); //Unit Ids on improvement work that do not exist
    }
  });
}

export async function addEventListener(projectId: string, card: projectCard) {
  $("#ongoingSeeDetailsButton" + projectId)
    .off("click")
    .on("click", async function () {
      const expandedDiv = document.getElementById(
        "expandedOngoingWorkCard" + projectId
      );
      if (expandedDiv && expandedDiv.style.display === "none") {
        card.showMoreInfo(
          `expandedOngoingWorkCard` + projectId,
          `ongoingSeeDetailsButton` + projectId
        );
      } else {
        card.closeMoreInfo(
          `expandedOngoingWorkCard` + projectId,
          `ongoingSeeDetailsButton` + projectId
        );
      }
    });

  $("#project-button-" + projectId)
    .off("click")
    .on("click", async function () {
      let tmp = await getObjectById("improvementWork", projectId);
      const currentStage = tmp.currentStage;
      if (currentStage === "Act") {
        loadProjectPageA(projectId);
      } else if (currentStage === "Study") {
        loadProjectPageS(projectId);
      } else if (currentStage === "Do") {
        loadProjectPageG(projectId);
      } else if (currentStage === "Plan") {
        loadProjectPageP(projectId);
      } else if (currentStage === "Finished") {
        loadProjectPageP(projectId);
      }
    });

  /*$(document).on("click", "#" + projectId, async function (event) {
      let tmp = await getObjectById("improvementWork", projectId);
      const currentStage = tmp.currentStage;
      const element = event.target as HTMLElement;
      if (element.classList.contains("moreInfoClass")) {
        const expandedDiv = document.getElementById(
          "expandedOngoingWorkCard" + projectId
        );
        if (expandedDiv && expandedDiv.style.display === "none") {
          card.showMoreInfo(
            `expandedOngoingWorkCard` + projectId,
            `ongoingSeeDetailsButton` + projectId
          );
        } else {
          card.closeMoreInfo(
            `expandedOngoingWorkCard` + projectId,
            `ongoingSeeDetailsButton` + projectId
          );
        }
      } else if (element.classList.contains("projectButton") || element.classList.contains("toSinglePage")) {
        document.removeEventListener("click", event.target);
        if (currentStage === "Act") {
          loadProjectPageA(projectId);
        } else if (currentStage === "Study") {
          loadProjectPageS(projectId);
        } else if (currentStage === "Do") {
          loadProjectPageG(projectId);
        } else if (currentStage === "Plan") {
          loadProjectPageP(projectId);
        } else if (currentStage === "Finished") {
          loadProjectPageP(projectId);
        }
      }
    });*/
}

const patientRiskLevelMapping: { [key: string]: string } = {
  Grön: "Green",
  Blå: "Blue",
  Gul: "Yellow",
  Röd: "Red",
};

const nolanStageMapping: { [key: string]: string } = {
  Planera: "Plan",
  Göra: "Do",
  Studera: "Study",
  Agera: "Act",
};

async function performSearch(searchBar: HTMLInputElement, container: Element) {
  let inputValue = searchBar.value;
  //make first character uppercase
  inputValue = inputValue.toLowerCase();
  console.log("Search query:", inputValue);
  // Remove each element
  projectCards = [];

  let query: { [key: string]: string | boolean } = {};
  query.completed = false;

  removeCardsFromDiv("div.projectCardCardDiv");

  if (inputValue !== "") {
    const matchedStage = Object.keys(nolanStageMapping).find((stage) =>
      stage.toLowerCase().includes(inputValue.toLowerCase())
    );

    if (matchedStage) {
      query.nolanStage = nolanStageMapping[matchedStage];
    } else {
      query.name = inputValue;
    }
  }
  filteredImprovementWorks = await getImprovementWorks(query);
  await getProjects(container, true, filteredImprovementWorks);
}
