import { onAuthStateChanged } from "firebase/auth";
import {
  auth,
  createUser,
  getImprovementWorkResponsible,
} from "./Database/database.js";
import { logout } from "./Database/authentication.js";
import {} from "./currentProjects/currentProjects.js";
import {
  loadProjectPageP,
  loadProjectPageA,
  loadProjectPageG,
  loadProjectPageS,
  loadInitiatePage
} from "./SidebarAndHeader/pageHandler.js";
import {
  getUsers,
  getSuggestions,
  getImprovementWorks,
  getUnits,
  getObjectById,
  getImprovementWorksForUser,
  addNoteToUser,
} from "../client/Database/database.js";

import { formatDate } from "./projectPage/projectPage.js";
function logoutUser(event: Event) {
  event.preventDefault();
  alert("logout");

  logout();
}

interface Project {
  ID: string;
  name: string;
  current_stage: string;
  content: string;
  unit_ID: string;
  date_started: Date;
}

let myIdeaCards: { [key: number]: IdeaCard } = {};

const cache: {[key: string]: any[]} = {};

class IdeaCard {
  private header: string;
  private department: string;
  private name: string;
  private description: string;
  private upvoteCount: number;
  private id: number;
  private suggestionID: string;
  private dateCreated: any;

  constructor(
    id: number,
    header: string,
    department: string,
    name: string,
    description: string,
    upvoteCount: number,
    suggestionID: string,
    dateCreated: any
  ) {
    this.header = header;
    this.department = department;
    this.name = name;
    this.description = description;
    this.upvoteCount = upvoteCount;
    this.id = id;
    this.suggestionID = suggestionID;
    this.dateCreated = dateCreated;
  }

  generateHTML(): string {
    return `<div class="totalExpandedIdeaCard">
          <div class="ideaCard">
              <div class="ideaCardBorder">
                  <div class="ideaCardImageHP">
                      <img src="../images/pinkFaded.png" class ="pinkFadedHP" alt="pinkFaded">
                  </div>
                  <div class="ideaCardMiddleText">
                      <p id="ideaCardHeader" class="ideaCardFont">${
                        this.header
                      }</p>
                      <p id="ideaCardDepartment" class="ideaCardFont">${
                        this.department
                      }</p>
                      <p id="ideaCardName" class="ideaCardFont">${
                        this.name
                      } ${formatDate(this.dateCreated)}</p>
                      <p id="ideaCardMoreInfo${
                        this.id
                      }" class="ideaCardFont moreInfoClass"> > Mer information</p>
                  </div>
              </div>
              <div class="ideaCardUpvotesHP">
              <div class="upvoteDivHP">
              <svg xmlns="http://www.w3.org/2000/svg" width="28" height="28" fill="currentColor" class="bi bi-caret-up upvoteSymbol" viewBox="0 0 16 16">
  <path d="M3.204 11h9.592L8 5.519 3.204 11zm-.753-.659 4.796-5.48a1 1 0 0 1 1.506 0l4.796 5.48c.566.647.106 1.659-.753 1.659H3.204a1 1 0 0 1-.753-1.659z"/>
</svg>              <div id ="upvoteNumber" class="ideaCardFont"> ${
      this.upvoteCount
    }</div>

              </div>
          </div>
          </div>
          <div id="expandedContent${
            this.id
          }" class= "expandedIdeaContent" style ="display : none;">
              <div class="expandedDescriptionText">
                  <p class="ideaCardFontComments" id="expandedHeader">Om ideen</p>
                  <p class="ideaCardFont" id="expandedDescription">${
                    this.description
                  }</p>
              </div>
              <div class="expandedCommentAndButton id = "expandedCommentAndButton${
                this.id
              }">
                  <div class="expandedComment">
                      <button class="ideaCardFontComments" id="commentsProjectCard">Kommentarer</button>
                  </div>

                  <div class="expandedButton">
                      <button class="initiateProjectButton" id="initiateProjectFromCard${
                        this.suggestionID
                      }">Initiera Förbättringsförslag</button>
                  </div>

                  <!-- commentsModal.html -->
                    <div id="commentsModal" class="modal">
                      <div class="modal-content">
                        <span class="close" id="closeModal">&times;</span>
                        <div id="existingComments">
                          <!-- Display existing comments here -->
                          <div class="comment-container">
                            <div class="comment">
                              <div class="comment-author">John Doe</div>
                              <div class="comment-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit.</div>
                            </div>
                        
                            <div class="comment">
                              <div class="comment-author">Jane Smith</div>
                              <div class="comment-text">Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.</div>
                            </div>

                            <div class="comment">
                              <div class="comment-author">Erik Svensson</div>
                              <div class="comment-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit.</div>
                            </div>
                          </div>
                        </div>
                        <form id="addCommentForm">
                          <input type="text" id="commentInput" name="commentInput">
                          <button type="button" id="addCommentBtn">Publicera</button>
                        </form>
                      </div>
                    </div>
              </div>
          </div>
      </div>`;
  }

  showMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId);
    if (blockElement) {
      blockElement.style.display = "flex";
    }

    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "^ Stäng detaljer";
    }
  }

  closeMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId) as HTMLSelectElement;
    if (blockElement) {
      blockElement.style.display = "none";
    }
    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "> Mer information";
    }
  }

  loadInitiatePage() {
    alert("The initiate project page should now be loaded");
  }
}
async function createNewCards(
  className: string,
  container: Element,
  signedInUserHsaID: string
) {
  let user = await getObjectById("users", signedInUserHsaID);
  let userSuggestions = await getSuggestions({ creatorHsaId: user.id });
  //If there exist filter use those instead to get matching suggestions rest should not need to be changed
  var counter = 0;
  userSuggestions.forEach(async (suggestion) => {
    var unit = await getObjectById("unit", suggestion.unitId);
    var user = await getObjectById("users", suggestion.creatorHsaId);
    createNewIdeaCard(
      suggestion.name,
      unit.name,
      user.firstName + " " + user.lastName,
      suggestion.content,
      suggestion.upvotes.length,
      className,
      container,
      counter,
      suggestion.id,
      suggestion.dateCreated
    );
    counter = counter + 1;
  });
  addListner(userSuggestions, container);
}

function addListner(cardArray: { [key: number]: IdeaCard }, element: Element) {
  element.addEventListener("click", function (event) {
    const target = event.target as HTMLElement;
    //Adds an eventlistener for the more info of the card
    if (target && target.className === "ideaCardFont moreInfoClass") {
      const cardId = parseInt(target.id.replace("ideaCardMoreInfo", ""), 10);
      if (cardArray[cardId]) {
        const card = cardArray[cardId];
        const expandedDiv = document.getElementById(`expandedContent${cardId}`);
        console.log(expandedDiv);
        if (expandedDiv) {
          if (expandedDiv.style.display === "none") {
            card.showMoreInfo(
              `expandedContent${cardId}`,
              `ideaCardMoreInfo${cardId}`
            );
          } else {
            card.closeMoreInfo(
              `expandedContent${cardId}`,
              `ideaCardMoreInfo${cardId}`
            );
          }
        } else {
          console.error("Expanded div not found");
        }
      }
    }

    if (target && target.className === "initiateProjectButton") {
      const cardId = target.id.replace("initiateProjectFromCard", "");
      loadInitiatePage(cardId);
    }
  });
}
async function createNewIdeaCard(
  header: string,
  department: string,
  name: string,
  description: string,
  upvoteCount: number,
  className: string,
  container: Element,
  ID: number,
  suggestionID: string,
  dateCreated: any
) {
  let card = new IdeaCard(
    ID,
    header,
    department,
    name,
    description,
    upvoteCount,
    suggestionID,
    dateCreated
  );
  let cardHTML = card.generateHTML();
  myIdeaCards[ID] = card;
  let div = document.createElement("div");
  div.className = className; // Set the class name(s) here
  div.innerHTML = cardHTML;
  container.appendChild(div);
}

export async function loadPage(listenersLoaded: boolean) {
  const container = document.querySelector("#container-fluid");

  if (!container) return; // Stop if the container isn't found
  fetch("homePage.html")
    .then((response) => response.text())
    .then(async (data) => {
      // Create a temporary container element to hold the loaded content
      const tempContainer = document.createElement("div");
      tempContainer.innerHTML = data;

      // Select the specific element with the class you want
      const specificElement = tempContainer.querySelector(".main-content");

      if (specificElement) {
        // Insert the specific element into the target element
        container.insertBefore(specificElement, container.firstChild);
      }

      const inputNotes = document.getElementById(
        "input-text-notes"
      ) as HTMLInputElement;
      if (inputNotes) {
        console.log("InputNotes exist");

        const tmp = localStorage.getItem("user");
        if (!tmp) {
          console.log("There is not a user in localStorage");
          return;
        }
        const uid = JSON.parse(tmp)[0].uid;
        const user = await getUsers({ uid: uid });
        const userNotes = user[0].notes;

        // Set the value of the input box
        inputNotes.value = userNotes || "";

        inputNotes.addEventListener("input", async () => {
          const inputNotesValue = inputNotes.value;

          try {
            if (user[0].id) {
              await addNoteToUser(user[0].id, inputNotesValue);
              console.log("Saved notes");
            }
          } catch (error) {
            console.log("Error saving notes", error);
          }
        });
      } else {
        console.log("No notes to be saved");
      }

      var tmp = localStorage.getItem("user");
      console.log(tmp);
      if (!tmp) {
        console.log("there is not an user in localStorage");
        return;
      }
      var user = JSON.parse(tmp)[0];

      const suggestionContainer = document.getElementById(
        "suggestion-container"
      );
      if (suggestionContainer) {
        await createNewCards(
          "HomePageIdeaCardDiv",
          suggestionContainer,
          user.id
        );

        addListner(myIdeaCards, container);
      }

      const nameHeaderElement = document.getElementById("user-name-header");

      let userFullName = user.firstName + " " + user.lastName;

      if (nameHeaderElement) {
        nameHeaderElement.textContent = userFullName;
      }
      const projectContainer = document.getElementById("project-container");
      if (projectContainer) {
        const startGetProjects = performance.now();
        getProjects(projectContainer, listenersLoaded, user.id);
        console.log(`getProjects duration: ${performance.now() - startGetProjects} ms`)
      }
      console.log("innan");
      const departmentContainer = document.getElementById(
        "department-container"
      );
      const departmentDiv = document.getElementById(
        "nolanAct"
      ) as HTMLDivElement;

      if (
        departmentContainer &&
        document.getElementById("nolanAct") &&
        departmentDiv.innerHTML.trim() === ""
      ) {
        console.log("anropar funktionen");
        await improvementWorkInfo(departmentContainer, user);
      }
    })
    .catch((error) => {
      console.error("Failed to load the external HTML file:", error);
    });
}

export async function addEventListener(projectId: string, card: projectCard) {
  $("#homepage-" + projectId)
    .off("click")
    .on("click", async function () {
      let tmp = await getObjectById("improvementWork", projectId);
      const currentStage = tmp.currentStage;
      if (currentStage === "Act") {
        loadProjectPageA(projectId);
      } else if (currentStage === "Study") {
        loadProjectPageS(projectId);
      } else if (currentStage === "Do") {
        loadProjectPageG(projectId);
      } else if (currentStage === "Plan") {
        loadProjectPageP(projectId);
      } else if (currentStage === "Finished") {
        loadProjectPageP(projectId);
      }
    });
}

const patientRiskLevelMapping: { [key: string]: string } = {
  Grön: "Green",
  Blå: "Blue",
  Gul: "Yellow",
  Röd: "Red",
};

const nolanStageMapping: { [key: string]: string } = {
  Planera: "Plan",
  Göra: "Do",
  Studera: "Study",
  Agera: "Act",
};
/*
async function performSearch(searchBar: HTMLInputElement, container: HTMLElement) {
  let inputValue = searchBar.value;
  //make first character uppercase
  inputValue = inputValue.toLowerCase();
  console.log("Search query:", inputValue);
  // Remove each element
  projectCards = [];

  let query: { [key: string]: string } = {};

  removeCardsFromDiv("div.projectCardCardDiv");
    
  if (inputValue !== "") {
    const matchedStage = Object.keys(nolanStageMapping).find((stage) =>
      stage.toLowerCase().includes(inputValue.toLowerCase())
    );

    if (matchedStage) {
      query.nolanStage = nolanStageMapping[matchedStage];
    } else {
      query.name = inputValue;
    }
  }
  filteredImprovementWorks = await getImprovementWorks(query);
  await getProjects(container, true, filteredImprovementWorks);
}
*/
async function improvementWorkInfo(departmentContainer: any, user: any) {
  //Get improvement works in the user's department
  const improvementWorks = await getImprovementWorks({
    unitId: user.unitId,
  });

  console.log("i funktionen");
  //Count number of improvement works in the department in each stage
  let improvementWorksPlan = 0;
  let improvementWorksDo = 0;
  let improvementWorkStudy = 0;
  let improvementWorksAct = 0;
  for (const improvementWork of improvementWorks) {
    if (improvementWork.currentStage == "Plan") {
      improvementWorksPlan += 1;
    } else if (improvementWork.currentStage == "Do") {
      improvementWorksDo += 1;
    } else if (improvementWork.currentStage == "Study") {
      improvementWorkStudy += 1;
    } else if (improvementWork.currentStage == "Act") {
      improvementWorksAct += 1;
    }
  }

  let improvementWorkContainer = document.getElementById(
    "numberOfImprovementWorks"
  );

  //Append a div displaying the number of improvement works in the user's department
  let improvementWorkContainerElement = document.createElement("p");
  improvementWorkContainerElement.innerHTML =
    "Antal pågående förbättringsarbeten i din avdelning: " +
    improvementWorks.length;
  if (
    improvementWorkContainer &&
    improvementWorkContainer.innerHTML.trim() === ""
  ) {
    improvementWorkContainer.appendChild(improvementWorkContainerElement);
  }

  //Append the text displaying the number of improvement works that are in the stage act
  let nolanAct = document.getElementById("nolanAct");
  let actElement = document.createElement("p");
  actElement.innerHTML = "Agera: " + improvementWorksAct;
  actElement.style.margin = "5px";
  if (nolanAct) {
    nolanAct.appendChild(actElement);
  }

  //Append the text displaying the number of improvement works that are in the stage plan
  let nolanPlan = document.getElementById("nolanPlan");
  let planElement = document.createElement("p");
  planElement.innerHTML = "Planera: " + improvementWorksPlan;
  planElement.style.margin = "5px";
  if (nolanPlan) {
    nolanPlan.appendChild(planElement);
  }

  //Append the text displaying the number of improvement works that are in the stage do
  let nolanDo = document.getElementById("nolanDo");
  let doElement = document.createElement("p");
  doElement.innerHTML = "Göra: " + improvementWorksDo;
  doElement.style.margin = "5px";
  if (nolanDo) {
    nolanDo.appendChild(doElement);
  }

  //Append the text displaying the number of improvement works that are in the stage study
  let nolanStudy = document.getElementById("nolanStudy");
  let studyElement = document.createElement("p");
  studyElement.innerHTML = "Studera: " + improvementWorkStudy;
  studyElement.style.margin = "5px";
  if (nolanStudy) {
    nolanStudy.appendChild(studyElement);
  }
}

export async function getProjects(
  selectElement: HTMLElement,
  listenersLoaded: boolean,
  userHsaId: string
) {
  let data: any[] = [];

  if(cache[userHsaId]) {
    const startGetCachedProjects = performance.now();
    data = cache[userHsaId];
    console.log(`getCachedProjects duration: ${performance.now() - startGetCachedProjects} ms`);
  } else {
    const startGetImprovementWorksForUser = performance.now();
    data = await getImprovementWorksForUser(userHsaId);
    cache[userHsaId] = data;
    console.log(`getImprovementWorksForUser duration: ${performance.now() - startGetImprovementWorksForUser} ms`);
  }

  let id = 0;
  data.forEach(async (project) => {
    let title = project.name;
    let department = await getObjectById("unit", project.unitId);
    let responsible = await getImprovementWorkResponsible(project.id);
    if (department) {
      let name = responsible.firstName + " " + responsible.lastName;
      let description = project.content;
      let i = id;
      let dateStarted = project.dateStarted;
      let card = new projectCard(
        i,
        name,
        department.name,
        title,
        description,
        project.id,
        dateStarted
      );
      projectCards[i] = card;
      const startGenerateHTML = performance.now();
      let cardHTML = card.generateHTML();
      console.log(`GenerateHTML duration ${performance.now() - startGenerateHTML} ms`);
      let div = document.createElement("div");
      div.className = "homePageProjectCardDiv"; // Set the class name(s) here
      div.innerHTML = cardHTML;
      if (!selectElement) return;
      selectElement.appendChild(div); // Stop if the container isn't found
      addEventListener(project.id, card);
      if (project.completed) {
        const datesDiv = document.getElementById(
          "ongoingCardDatesHomePage" + project.id
        );
        if (datesDiv) {
          datesDiv.innerText += " till " + formatDate(project.endDate);
        }
      }
      const stageDiv = document.getElementById(
        "currentStageLetterHomePage" + project.id
      );
      if (stageDiv) {
        if (project.completed) {
          stageDiv.innerText = "Avslutat";
          stageDiv.style.fontSize = "25px";
          stageDiv.style.marginLeft = "5px";
          stageDiv.style.marginTop = "30px";
        } else if (project.currentStage == "Plan") {
          stageDiv.innerHTML = "P";
        } else if (project.currentStage == "Do") {
          stageDiv.innerHTML = "G";
        } else if (project.currentStage == "Study") {
          stageDiv.innerText = "S";
        } else if (project.currentStage == "Act") {
          stageDiv.innerText = "A";
        }
      }
      id++;
    } else {
      console.log(project.unitId + " is not a unit"); //Unit Ids on improvement work that do not exist
    }
  });
}

let projectCards: { [key: number]: projectCard } = {};

class projectCard {
  private name: string; // Projectowner
  private department: string;
  private title: string;
  private description: string;
  private id: number;
  private projectId: string;
  private dateStarted: any;

  constructor(
    id: number,
    name: string,
    department: string,
    title: string,
    description: string,
    projectId: string,
    dateStarted: string
  ) {
    this.name = name;
    this.department = department;
    this.title = title;
    this.description = description;
    this.id = id;
    this.projectId = projectId;
    this.dateStarted = dateStarted;
    //this.endDate = endDate;
  }

  showMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId);
    if (blockElement) {
      blockElement.style.display = "flex";
    }

    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "^ Stäng detaljer";
    }
  }

  closeMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId) as HTMLSelectElement;
    if (blockElement) {
      blockElement.style.display = "none";
    }
    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "> Mer information";
    }
  }

  loadInitiatePage() {
    alert("The initiate project page should now be loaded");
  }

  generateHTML(): string {
    return `<div class="homeTotalExpandedOngoingWorkCard">
        <div style="cursor: pointer;" class="flex-container ongoingCard" id="homepage-${
          this.projectId
        }">
            <div class="ongoingCardImageContainer toSinglePage">
              <div id="borderPinkFaded">
                <div class = "flex-box ongoingCardImage toSinglePage">
                  <div id="currentStageLetterHomePage${
                    this.projectId
                  }" style="font-size: 50px; font-weight: bolder;
                  color: #845380;
                  position: inherit; margin-left: 35px; margin-top: 10px;"></div>
                </div>
              </div>
            </div>
            <div class="div ongoingCardInfo toSinglePage">
                <p class="toSinglePage" id="ongoingCardIdeaTitle">${
                  this.title
                }</p>
                <p class="toSinglePage" id="ongoingCardDepartment">${
                  this.department
                }</p>
                <p class="toSinglePage" id="ongoingCardManager">${this.name}</p>
                <p class="toSinglePage" style="font-style: italic; margin-top: 5px;" id="ongoingCardDatesHomePage${
                  this.projectId
                }">${formatDate(this.dateStarted)} </p>
            </div>

            <!--<button id="homepage-project-button-${
              this.projectId
            }" data-project-id="${
      this.id
    }" class="projectButton" role="button">Gå till projektsidan</button>-->


            <div class="container ongoingSeeCardDetails toSinglePage">
                <div class="ongoingSeeCardDetailsInner">
                    
                </div>
            </div>
        </div> 
        <div class = "expandedOngoingWorkCard" id="homepage-expandedOngoingWorkCard${
          this.projectId
        }" style="display: none;">
            <div class ="homepage-expandedTextOngoingWorkCard">
            ${this.description}
            </div>
        </div>
        </div> `;
  }
}

//<p id="homepage-ongoingSeeDetailsButton${this.projectId}" class = "moreInfoClass">> Mer information</p>
