import { create } from "domain";
import {
  getObject,
  getObjectById,
  getSuggestions,
  getUsers,
  getHospitals,
  getUnits,
  upvoteSuggestion,
} from "../Database/database.js";
import { formatDate } from "../projectPage/projectPage.js";
import { loadInitiatePage } from "../SidebarAndHeader/pageHandler.js";
import { hostname } from "os";
import { getEventListeners } from "events";
//import {loadInitiatePage} from "../SidebarAndHeader/pageHandler.js";

let myIdeaCards: { [key: number]: IdeaCard } = {};

// Initialize filtered suggestions as empty array (potentially change to get all suggestions
let filteredSuggestions: any[] = [];

class IdeaCard {
  async hasUserUpvoted() {
    const tmp = localStorage.getItem("user");
    if (!tmp) {
      console.log("No user in localStorage");
      return false;
    }
    const user = JSON.parse(tmp)[0];

    try {
      let suggestion = await await getObjectById(
        "suggestion",
        this.suggestionID
      );

      // Check if suggestion and suggestion are defined
      if (!suggestion || suggestion.length === 0) {
        console.error("Suggestion not found or is invalid");
        return false;
      }

      if (suggestion.upvotes.includes(user.id)) {
        return true;
      }
    } catch (error) {
      console.error("Error in hasUserUpvoted: ", error);
      return false;
    }
  }

  async addUpvote() {
    var tmp = localStorage.getItem("user");
    if (!tmp) {
      console.log("there is not a user in localStorage");
      return;
    }
    var user = JSON.parse(tmp)[0];

    let upvoted = await upvoteSuggestion(this.suggestionID, user.id);

    if (upvoted) {
      this.upvoteCount++;
    } else {
      this.upvoteCount--;
    }
    return upvoted;
  }

  private header: string;
  private department: string;
  private name: string;
  private description: string;
  public upvoteCount: number;
  private id: number;
  private suggestionID: string;
  private dateCreated: any;

  constructor(
    id: number,
    header: string,
    department: string,
    name: string,
    description: string,
    upvoteCount: number,
    suggestionID: string,
    dateCreated: any
  ) {
    this.header = header;
    this.department = department;
    this.name = name;
    this.description = description;
    this.upvoteCount = upvoteCount;
    this.id = id;
    this.suggestionID = suggestionID;
    this.dateCreated = dateCreated;
  }
  // <img src="images/upvoteIcon.png" class ="upvoteIcon" alt="upvoteIcon">
  async generateHTML() {
    const userHasUpvoted = await this.hasUserUpvoted();

    const upvoteColor = userHasUpvoted ? "purple" : "white";
    const strokeColor = userHasUpvoted ? "purple" : "grey";

    return `<div class="totalExpandedIdeaCard">
          <div class="ideaCard">
              <div class="ideaCardBorder">
                  <div class="ideaCardImage">
                      <img src="../images/pinkFaded.png" class ="pinkFaded" alt="pinkFaded">
                  </div>
                  <div class="ideaCardMiddleText">
                      <p id="ideaCardHeader" class="ideaCardFont">${
                        this.header
                      }</p>
                      <p id="ideaCardDepartment" class="ideaCardFont">${
                        this.department
                      }</p>
                      <p id="ideaCardName" class="ideaCardFont">${
                        this.name
                      }  ${formatDate(this.dateCreated)}</p>
                      <p id="ideaCardMoreInfo${
                        this.id
                      }" class="ideaCardFont moreInfoClass"> > Mer information</p>
                  </div>
                  <div class="ideaCardUpvotes">
                  
                  <div class="upvoteDiv">
  <button class="upvoteButton" id="upvoteButton${this.id}">
    <svg xmlns="http://www.w3.org/2000/svg" width="28" height="28" stroke="${strokeColor}" fill="${upvoteColor}" class="bi bi-caret-up upvoteSymbol" viewBox="0 0 24 24">
      <path d="M18 7.2071C18 6.09347 16.6536 5.53575 15.8661 6.32321L6.32321 15.8661C5.53576 16.6536 6.09346 18 7.2071 18H16.25C17.2165 18 18 17.2165 18 16.25V7.2071Z" transform="rotate(225 12 12)"/>
    </svg>
    <div class="ideaCardFont upvoteNumber">${this.upvoteCount}</div>
  </button>
</div>

                  </div>
              </div>
              </div>
    
          </div>
          <div id="expandedContent${
            this.id
          }" class= "expandedIdeaContent" style ="display : none;">
              <div class="expandedDescriptionText">
                  <p class="ideaCardFontComments" id="expandedHeader">Om ideen</p>
                  <p class="ideaCardFont" id="expandedDescription">${
                    this.description
                  }</p>
              </div>
              <div class="expandedCommentAndButton id = "expandedCommentAndButton${
                this.id
              }">
                  <div class="expandedComment">
                      <button class="ideaCardFontComments" id="commentsProjectCard">Kommentarer</button>
                  </div>

                  <div class="expandedButton">
                      <button class="initiateProjectButton" id="initiateProjectFromCard${
                        this.suggestionID
                      }">Initiera Förbättringsförslag</button>
                  </div>

                  <!-- commentsModal.html -->
                    <div id="commentsModal" class="modal">
                      <div class="modal-content">
                        <span class="close" id="closeModal">&times;</span>
                        <div id="existingComments">
                          <!-- Display existing comments here -->
                          <div class="comment-container">
                            <div class="comment">
                              <div class="comment-author">John Doe</div>
                              <div class="comment-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit.</div>
                            </div>
                        
                            <div class="comment">
                              <div class="comment-author">Jane Smith</div>
                              <div class="comment-text">Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.</div>
                            </div>

                            <div class="comment">
                              <div class="comment-author">Erik Svensson</div>
                              <div class="comment-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit.</div>
                            </div>
                          </div>
                        </div>
                        <form id="addCommentForm">
                          <input type="text" id="commentInput" name="commentInput">
                          <button type="button" id="addCommentBtn">Publicera</button>
                        </form>
                      </div>
                    </div>
              </div>
          </div>
      </div>`;
  }

  openCommentsModal() {
    // Retrieve existing comments and display in the modal
    const existingCommentsContainer =
      document.getElementById("existingComments");
    // Logic to fetch and display existing comments goes here...

    // Show the comments modal
    const modal = document.getElementById("commentsModal");
    if (modal) {
      modal.classList.add("show-modal");
      modal.style.display = "block";
      // Add click event listener for the close button
      const closeButton = modal.querySelector("#closeModal");
      if (closeButton) {
        closeButton.addEventListener("click", () => {
          this.closeCommentsModal();
        });
      }
      const publishButton = modal.querySelector("#addCommentBtn");
      if (publishButton) {
        publishButton.addEventListener("click", () => {
          alert(
            "Kul att du vill kommentera det här förbättringsförslaget! Vi har ännu inte implementerat kommentarsfältet så kommentarerna du ser är hårdkodade av oss."
          );
        });
      }
    }
  }

  closeCommentsModal() {
    // Close the comments modal
    const modal = document.getElementById("commentsModal");
    if (modal) {
      modal.style.display = "none";
      modal.classList.remove("show-modal");
    }
  }

  showMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId);
    if (blockElement) {
      blockElement.style.display = "flex";
    }

    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "^ Stäng detaljer";
    }
  }

  closeMoreInfo(blockId: string, thisTextId: string): void {
    const blockElement = document.getElementById(blockId) as HTMLSelectElement;
    if (blockElement) {
      blockElement.style.display = "none";
    }
    const textElement = document.getElementById(thisTextId);
    if (textElement) {
      textElement.innerText = "> Mer information";
    }
  }

  loadInitiatePage() {
    alert("The initiate project page should now be loaded");
    window.location.href = "../client/projectPage/projectPageP.html";
  }
}
async function createNewCards(
  className: string,
  container: Element,
  suggestions: any[]
) {
  var counter = 1;
  suggestions.forEach(async (suggestion) => {
    var unit = await getObjectById("unit", suggestion.unitId);
    var user = await getObjectById("users", suggestion.creatorHsaId);
    console.log(suggestion);
    createNewIdeaCard(
      suggestion.name,
      unit.name,
      user.firstName + " " + user.lastName,
      suggestion.content,
      suggestion.upvotes.length,
      className,
      container,
      counter,
      suggestion.id,
      suggestion.dateCreated
    );
    counter = counter + 1;
  });
}

async function createNewIdeaCard(
  header: string,
  department: string,
  name: string,
  description: string,
  upvoteCount: number,
  className: string,
  container: Element,
  ID: number,
  suggestionID: string,
  dateCreated: any
) {
  let card = new IdeaCard(
    ID,
    header,
    department,
    name,
    description,
    upvoteCount,
    suggestionID,
    dateCreated
  );
  let cardHTML = await card.generateHTML();
  myIdeaCards[ID] = card;
  let div = document.createElement("div");
  div.className = className; // Set the class name(s) here
  div.innerHTML = cardHTML;

  // Add click event listener for "Kommentarer" button
  const commentsButton = div.querySelector(`#commentsProjectCard`);
  if (commentsButton) {
    commentsButton.addEventListener("click", (event) => {
      // Handle the click event, e.g., open the comments modal
      card.openCommentsModal();
    });
  }

  container.appendChild(div);
}

export function testFunc() {
  console.log("Test func works");
}

export async function loadPage() {
  const container = document.querySelector("#container-fluid");
  if (!container) return; // Stop if the container isn't found
  const scriptElement = document.createElement("script");
  scriptElement.src = "../dist/ideaBank.bundle.js";
  container.appendChild(scriptElement);

  // Initialize empty lists of checked hospital and unit ids
  let checkedHospitalIds: string[] = [];
  let checkedUnitIds: string[] = [];

  fetch("ideaBank/ideaBank.html")
    .then((response) => response.text())
    .then((data) => {
      // Create a temporary container element to hold the loaded content
      const tempContainer = document.createElement("div");
      tempContainer.innerHTML = data;

      // Select the specific element with the class you want
      const specificElement = tempContainer.querySelector(".filterBar");

      const createHospitalCheckBoxContainer = tempContainer.querySelector(
        "#hospitalSelection"
      ) as HTMLSelectElement;
      const createUnitCheckBoxContainer = tempContainer.querySelector(
        "#unitSelection"
      ) as HTMLSelectElement;

      if (createHospitalCheckBoxContainer) {
        createHospitalCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes = createHospitalCheckBoxContainer.querySelectorAll(
              'input[type="checkbox"]'
            );

            // Empty list of checked hospitals
            checkedHospitalIds = [];

            // Array to store IDs of checked checkboxes
            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedHospitalIds.push(inputCheckbox.id);
              }
            });

            // Log all checked hospital IDs
            filteredSuggestions = await getFilteredSuggestions(
              undefined,
              checkedHospitalIds
            );
            removeCardsFromDiv("div.IdeaCardDiv");
            createNewCards("IdeaCardDiv", container, filteredSuggestions);

            // Future change: Only clear the unitIds for unchecked hospitals!
            clearContent("unitSelection");
            loadUnits(checkedHospitalIds);
          }
        );
      }

      if (createUnitCheckBoxContainer) {
        createUnitCheckBoxContainer.addEventListener(
          "change",
          async function (event) {
            const checkboxes = createUnitCheckBoxContainer.querySelectorAll(
              'input[type="checkbox"]'
            );

            // Empty list of checked units
            checkedUnitIds = [];

            checkboxes.forEach((checkbox) => {
              // Ensure the checkbox is an HTMLInputElement
              const inputCheckbox = checkbox as HTMLInputElement;

              // Check if the checkbox is checked and add its ID to the array
              if (inputCheckbox.checked) {
                checkedUnitIds.push(inputCheckbox.id);
              }
            });

            // Log all checked unit IDs
            filteredSuggestions = await getFilteredSuggestions(checkedUnitIds);
            removeCardsFromDiv("div.IdeaCardDiv");
            createNewCards("IdeaCardDiv", container, filteredSuggestions);
          }
        );
      }

      if (specificElement) {
        // Insert the specific element into the target element
        container.insertBefore(specificElement, container.firstChild);
      }
    })
    .catch((error) => {
      console.error("Failed to load the external HTML file:", error);
    });
  myIdeaCards = {};
  filteredSuggestions = await getSuggestions({});
  console.log("Filtered suggestions: ", filteredSuggestions);
  removeCardsFromDiv("div.IdeaCardDiv");
  await createNewCards("IdeaCardDiv", container, filteredSuggestions);
  addListner(myIdeaCards, container);

  const searchBar = document.getElementById("searchBarId") as HTMLInputElement;
  searchBar.addEventListener("keydown", function (event) {
    if (event.key === "Enter") {
      performSearch(searchBar);
    }
  });
  loadHospitals();
}

async function getFilteredSuggestions(
  checkedUnitIds?: string[],
  checkedHospitalIds?: string[]
) {
  let suggestions: any[] = [];

  if (checkedUnitIds?.length) {
    for (const checkedUnitId of checkedUnitIds) {
      const allSuggestionsOfUnit = await getSuggestions({
        unitId: checkedUnitId,
      });
      suggestions = suggestions.concat(allSuggestionsOfUnit);
    }
    return suggestions;
  } else if (checkedHospitalIds?.length) {
    for (const checkedHospital of checkedHospitalIds) {
      const allSuggestionsOfhospital = await getSuggestions({
        departmentId: checkedHospital,
      });
      suggestions = suggestions.concat(allSuggestionsOfhospital);
    }
    console.log("efter");
    return suggestions;
  } else {
    suggestions = await getSuggestions({});
    return suggestions;
  }
}

export async function loadHospitals() {
  const selectHospitalHTML = $("#hospitalSelection"); // loads the select type from html
  const allHospitals = await getHospitals({});
  console.log(allHospitals);
  allHospitals.forEach((hospital) => {
    const generatedElement = document.createRange().createContextualFragment(
      `
    <div class="form-check">
      <input class="form-check-input" type="checkbox" value="" id="` +
        hospital.id +
        `">
      <label class="form-check-label" for="flexCheckDefault">` +
        hospital.name +
        `</label>
    </div>`
    );
    selectHospitalHTML.append(generatedElement);
    console.log("Adding HTML for for hospital ", hospital.id);
  });
}

export async function loadUnits(checkedHospitalIds: string[]) {
  const selectUnitHTML = $("#unitSelection"); // loads the select type from html
  let allUnits: any[] = [];

  if (checkedHospitalIds.length != 0) {
    for (const checkedHospitalId of checkedHospitalIds) {
      const allUnitsOfHospital = await getUnits({
        hospitalId: checkedHospitalId,
      });
      allUnits = allUnits.concat(allUnitsOfHospital);
    }
  }
  allUnits.forEach((unit) => {
    const generatedElement = document.createRange().createContextualFragment(
      `
    <div class="form-check">
      <input class="form-check-input" type="checkbox" value="" id="` +
        unit.id +
        `">
      <label class="form-check-label" for="flexCheckDefault">` +
        unit.name +
        `</label>
    </div>`
    );
    selectUnitHTML.append(generatedElement);
    console.log("Adding HTML for for unit ", unit.id);
  });
}

export function clearContent(id: string) {
  const container = document.getElementById(id);
  while (container!.firstChild) {
    container!.removeChild(container!.firstChild);
  }
}

async function addListner(
  cardArray: { [key: number]: IdeaCard },
  element: Element
) {
  element.addEventListener("click", async function (event) {
    const target = event.target as HTMLElement;
    //Adds an eventlistener for the more info of the card
    if (target && target.className === "ideaCardFont moreInfoClass") {
      console.log("fel Div");

      const cardId = parseInt(target.id.replace("ideaCardMoreInfo", ""), 10);
      if (cardArray[cardId]) {
        const card = cardArray[cardId];
        const expandedDiv = document.getElementById(`expandedContent${cardId}`);
        if (expandedDiv) {
          if (expandedDiv.style.display === "none") {
            card.showMoreInfo(
              `expandedContent${cardId}`,
              `ideaCardMoreInfo${cardId}`
            );
          } else {
            card.closeMoreInfo(
              `expandedContent${cardId}`,
              `ideaCardMoreInfo${cardId}`
            );
          }
        } else {
          console.error("Expanded div not found");
        }
      }
    }
    
    if (target && target.className === "initiateProjectButton") {
      const cardId = target.id.replace("initiateProjectFromCard", "");
      loadInitiatePage(cardId);
    }

    const upvoteButton = target.closest('.upvoteButton') as HTMLButtonElement | null;
    if (upvoteButton) {
      const cardIdStr = upvoteButton.id.replace("upvoteButton", "");
      const cardId = parseInt(cardIdStr, 10);

      // Check if the cardId is a valid number
      if (!isNaN(cardId) && myIdeaCards[cardId]) {
        const card = myIdeaCards[cardId];
        let upvoted = await card.addUpvote();
        console.log("upvoted :", upvoted);

        const svg = upvoteButton.querySelector("svg");
        if (svg) {
          if (upvoted) {
            svg.setAttribute("fill", "purple");
            svg.setAttribute("stroke", "purple");
          } else {
            svg.setAttribute("stroke", "grey");
            svg.setAttribute("fill", "white");
          }

          const upvoteNumberElement =
            upvoteButton.querySelector(".upvoteNumber");
          if (upvoteNumberElement) {
            upvoteNumberElement.textContent = card.upvoteCount.toString(); // Assuming card.upvoteCount holds the new count
          }
        }
      }
    }
  });
}

export function removeCardsFromDiv(divName: string) {
  //Remove current elements
  const elementsToRemove = document.querySelectorAll(divName);
  // Remove each element
  elementsToRemove.forEach((element) => {
    element.remove();
  });
  myIdeaCards = {};
}

async function performSearch(searchBar: HTMLInputElement) {
  const inputValue = searchBar.value;
  console.log("Search query:", inputValue);

  const container = document.querySelector("#container-fluid");
  if (!container) return; // Stop if the container isn't found

  removeCardsFromDiv("div.IdeaCardDiv");

  if (inputValue != "") {
    const suggestions = await getSuggestions({});
    let suggestionName = false;

    for (const suggestion of suggestions) {
      // Check if suggestion name matches input (fuzzy)
      if (
        suggestion.name.toLowerCase() >= inputValue.toLowerCase() &&
        suggestion.name.toLowerCase() <= inputValue.toLowerCase() + "~"
      ) {
        suggestionName = true;
        filteredSuggestions = await getSuggestions({ name: suggestion.name });
        createNewCards("IdeaCardDiv", container, filteredSuggestions);
      }
    }

    // Check if creator name matches input (fuzzy)
    if (!suggestionName) {
      const users = await getUsers({});
      const spaceIndex: number = inputValue.indexOf(" ");
      //If the search is for first and last name
      if (spaceIndex !== -1) {
        const firstName: string = inputValue
          .substring(0, spaceIndex)
          .toLowerCase();
        const lastName: string = inputValue
          .substring(spaceIndex + 1)
          .toLowerCase();
        for (const user of users) {
          if (
            user.firstName.toLowerCase() >= firstName &&
            user.firstName.toLowerCase() <= firstName + "~"
          ) {
            if (
              user.lastName.toLowerCase() >= lastName &&
              user.lastName.toLowerCase() <= lastName + "~"
            ) {
              filteredSuggestions = await getSuggestions({
                creatorHsaId: user.id,
              });
              createNewCards("IdeaCardDiv", container, filteredSuggestions);
            }
          }
        }
      } else {
        //If the search is for only first or last name
        for (const user of users) {
          if (
            user.firstName.toLowerCase() >= inputValue.toLowerCase() &&
            user.firstName.toLowerCase() <= inputValue.toLowerCase() + "~"
          ) {
            filteredSuggestions = await getSuggestions({
              creatorHsaId: user.id,
            });
            createNewCards("IdeaCardDiv", container, filteredSuggestions);
          } else if (
            user.lastName.toLowerCase() >= inputValue.toLowerCase() &&
            user.lastName.toLowerCase() <= inputValue.toLowerCase() + "~"
          ) {
            filteredSuggestions = await getSuggestions({
              creatorHsaId: user.id,
            });
            createNewCards("IdeaCardDiv", container, filteredSuggestions);
          }
        }
      }
    }
  } else {
    filteredSuggestions = await getSuggestions({});
    createNewCards("IdeaCardDiv", container, filteredSuggestions);
  }
  addListner(myIdeaCards, container);
}
