import {
  getObjectById,
  getUsers,
  getHospitals,
  getUnits,
  createImprovementWork,
  deleteSuggestion,
} from "../Database/database.js";
import { loadCurrentProjects } from "../SidebarAndHeader/pageHandler.js";

//Globala varibler för projekledare & Users
let selectedEmployees: string[] = [];

export async function loadEmployees() {
  const selectProjectLeaderHTML = $('#projectLeader'); // loads the select type from html
  const selectEmployeeHTML = $('#selectEmployee'); // loads the select type from html
  const allUsers = await getUsers({});
  const allProjectLeaders = await getUsers({}); //here you can put constraints

  allUsers.forEach((user) => {
    const optionElement = $("<option>")
      .addClass("ideaCardFont")
      .val(user.id)
      .text(`${user.firstName}` + ` ` + `${user.lastName}`);
    selectEmployeeHTML.append(optionElement);
  });

  allProjectLeaders.forEach((user) => {
    const optionElement = $("<option>")
      .addClass("ideaCardFont")
      .val(user.id)
      .text(`${user.firstName}` + ` ` + `${user.lastName}`);
    selectProjectLeaderHTML.append(optionElement);
  });

  const selectHospitalHTML = $("#selectHospital"); // loads the select type from html
  const selectUnitHTML = $("#selectDepartment"); // loads the select type from html
  const allHospitals = await getHospitals({});
  const allUnits = await getUnits({}); //here you can put constraints
  console.log(allHospitals);
  allHospitals.forEach((user) => {
    const optionElement = $("<option>")
      .addClass("ideaCardFont")
      .val(user.id)
      .text(user.name);
    selectHospitalHTML.append(optionElement);
  });
  allUnits.forEach((user) => {
    const optionElement = $("<option>")
      .addClass("ideaCardFont")
      .val(user.id)
      .text(user.name);
    selectUnitHTML.append(optionElement);
  });
}

export function assignedEmployer() {
  const employeeList = $("#selectEmployee");
  const selectedBox = $("#selectedEmployees");

  const selectedOption = employeeList.find("option:selected");
  const selectedText = selectedOption.text();

  selectedBox.append(
    createAssignedEmployeeDesign(selectedText, selectedOption, employeeList)
  );
  selectedEmployees.push(String(selectedOption.val()));
  employeeList.find("option:selected").remove();
  employeeList.prop("selectedIndex", 0);
}

export function getSelectedEmployees() {
  return selectedEmployees;
}
export function setSelectedEmployeesEmpty() {
  selectedEmployees = [];
}

function createAssignedEmployeeDesign(
  s: string,
  x: JQuery<HTMLElement>,
  employeeList: JQuery<HTMLElement>
) {
  const container = $("<div>").addClass("selectedEmployeeBubble"); // Create a div element and add a class
  const textSpan = $("<span>").addClass("ideaCardFont").text(s); // Create a span element, set text, and add a class
  const deleteButton = $("<button>").addClass("employeeDeleteButton").text("X"); // Create a button element, set text, and add a class

  deleteButton.on("click", function () {
    container.remove();
    employeeList.append(x);
    employeeList.prop("selectedIndex", 0);

    //removes from global list
    const index = selectedEmployees.indexOf(String(x.val()));
    if (index !== -1) {
      selectedEmployees.splice(index, 1);
    }
  });

  container.append(textSpan, deleteButton); // Append the span and button to the div container
  return container;
}

export async function loadPage(id: string) {
  const container = document.querySelector("#container-fluid");

  if (!container) return; // Stop if the container isn't found
  const linkelementCSS = document.createElement("link");
  linkelementCSS.rel = "styleSheet";
  linkelementCSS.href = "initiateForbattringsarbete/initiateIdea.css";
  document.head.appendChild(linkelementCSS);

  const linkQuestionCSS = document.createElement("link");
  linkQuestionCSS.rel = "styleSheet";
  linkQuestionCSS.href = "Make_Suggestion_Code/makeSuggestion.css";
  document.head.appendChild(linkQuestionCSS);

  const linkActivitycardCSS = document.createElement("link");
  linkActivitycardCSS.rel = "stylesheet";
  linkActivitycardCSS.href = "Activity_card_code/activitity_card.css";
  document.head.appendChild(linkActivitycardCSS);

  const scriptElement = document.createElement("script");
  scriptElement.src = "../dist/initiateIdea.bundle.js";
  container.appendChild(scriptElement);

  const generatedHTML = await generateQuestionHTML(id);
  const generatedElement = document
    .createRange()
    .createContextualFragment(generatedHTML);

  fetch("initiateForbattringsarbete/initiateIdea.html")
    .then((response) => response.text())
    .then((data) => {
      // Create a temporary container element to hold the loaded content
      const tempContainer = document.createElement("div");
      tempContainer.innerHTML = data;
      // Select the specific element with the class you want
      const specificElement = tempContainer.querySelector(
        "#initiateIdeaWholePage"
      );
      //Eventlistenerbuttons:
      const createButton = tempContainer.querySelector(
        "#createProjectFromSuggestionButton"
      ) as HTMLButtonElement;
      const selectEmployeeChangeSection = tempContainer.querySelector(
        "#selectEmployee"
      ) as HTMLButtonElement;

      if (createButton) {
        createButton.addEventListener("click", () => createProject(id));
      } else {
        console.error("Error: 'createButton' not found");
      }
      if (selectEmployeeChangeSection) {
        selectEmployeeChangeSection.addEventListener(
          "change",
          assignedEmployer
        );
      } else {
        console.error("Error: 'changebutton' not found");
      }

      if (specificElement) {
        console.error("spec element ok");
        container.insertBefore(createButton, container.firstChild);
        // Insert the generated element before the specific element
        container.insertBefore(generatedElement, container.firstChild);
        // Insert the specific element into the target element
        container.insertBefore(specificElement, container.firstChild);

        selectedEmployees = []; //clears the global employee list
        loadEmployees();
      }
    })
    .catch((error) => {
      console.error("Failed to load the external HTML file:", error);
    });
}
function createProject(id: string) {
  const inputSelectProjectleaderInput = document.getElementById(
    "projectLeader"
  ) as HTMLSelectElement;

  const selectDepartment = document.getElementById(
    "selectHospital"
  ) as HTMLSelectElement;
  const selectUnit = document.getElementById(
    "selectDepartment"
  ) as HTMLSelectElement;

  const inputBoxFormNameInput = document.getElementById(
    "inputBoxFormNameInput"
  ) as HTMLSelectElement;
  const inputBoxFormAreaInput = document.getElementById(
    "inputBoxFormAreaInput"
  ) as HTMLSelectElement;
  const inputBoxFormContentInput = document.getElementById(
    "inputBoxFormContentInput"
  ) as HTMLSelectElement;
  const inputBoxFormPurposeInput = document.getElementById(
    "inputBoxFormPurposeInput"
  ) as HTMLSelectElement;
  const inputBoxFormMeasureInput = document.getElementById(
    "inputBoxFormMeasureInput"
  ) as HTMLSelectElement;
  const inputBoxFormIdeasInput = document.getElementById(
    "inputBoxFormIdeasInput"
  ) as HTMLSelectElement;
  const inputBoxFormResourcesInput = document.getElementById(
    "inputBoxFormResourcesInput"
  ) as HTMLSelectElement;
  const hospitalId = document.getElementById(
    "selectHospital"
  ) as HTMLSelectElement;

  // alert("Kul att du försöker! Detta är inte klart ännu... \n Däremot är värdena följande: \n"
  //+ inputSelectProjectleaderInput.options[inputSelectProjectleaderInput.selectedIndex].value + "\n"
  //+ selectDepartment.options[selectDepartment.selectedIndex].value + "\n"
  //+ selectUnit.options[selectUnit.selectedIndex].value + "\n"
  //+ selectedEmployees + "\n"
  //+ inputBoxFormNameInput.value + "\n"
  //+ inputBoxFormAreaInput.value + "\n"
  //+ inputBoxFormContentInput.value + "\n"
  //+ inputBoxFormPurposeInput.value + "\n"
  //+ inputBoxFormMeasureInput.value + "\n"
  //+ inputBoxFormIdeasInput.value + "\n" );

  try {
    {
      createImprovementWork(
        null,
        inputBoxFormNameInput.value,
        "Plan",
        inputBoxFormContentInput.value,
        selectUnit.value, //unit id knows wich hospital it is refered to
        inputBoxFormPurposeInput.value,
        "Green",
        inputBoxFormAreaInput.value,
        inputSelectProjectleaderInput.options[
          inputSelectProjectleaderInput.selectedIndex
        ].value,
        selectedEmployees,
        inputBoxFormResourcesInput.value,
        hospitalId.value,
        inputBoxFormContentInput.value,
        inputBoxFormPurposeInput.value,
        undefined,
        inputBoxFormMeasureInput.value,
        undefined,
        inputBoxFormIdeasInput.value
      );
    }
    // Call the createSuggestion function with the obtained values
    loadCurrentProjects();
  } catch {
    alert("Error creating");
  }
  deleteSuggestion(id);
}

async function generateQuestionHTML(suggestionID: string): Promise<string> {
  const suggestionData = await getObjectById("suggestion", suggestionID); //gets suggestion from database
  return (
    `
  <div class="filter-section">
  <div class="makeSuggestionFilterButtons">
    <select name="department" id="selectHospital" class="filterButtons"> </select>

    <select name="section" id="selectDepartment" class="filterButtons">
    </select>

  </div>
</div>


<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">Titel *</p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormName"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormNameInput"
        placeholder="Vad vill du kalla ditt förslag?"
        value = "` +
    suggestionData.name +
    `"
        required
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">
      Vilket är området för förbättringen? *
    </p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormArea"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormAreaInput"
        placeholder="Berätta om område ditt förslag hade förbättrat."
        value = "` +
    suggestionData.area +
    `"
        required
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText" id="middleShadowText">
    <p id="suggestionQuestion" class="activityCardFont">Bakgrund *</p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormContent"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormContentInput"
        placeholder="Beskriv problemet"
        value = "` +
    suggestionData.content +
    `"
        required
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">
    Vad är målet med förbättringen? *
    </p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormPurpose"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormPurposeInput"
        placeholder="Vad ska åstadkommas och varför? Kom ihåg att få med patientperspektivet"
        value = "` +
    suggestionData.purpose +
    `"
        required
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">
      Hur ska det mätas?
    </p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormMeasure"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormMeasureInput"
        placeholder="Hur vet vi att en förändring är en förbättring?"
        value = "` +
    suggestionData.measure +
    `"
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">Förslag på lösning</p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormIdeas"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormIdeasInput"
        placeholder="Vilka förändringar kan prövas för att uppnå en förbättring?"
        value = "` +
    suggestionData.ideas +
    `"
      />
    </form>
  </div>
</div>
</div>

<div class="activityCard" id="suggestionCard">
<div class="activityCardBorder">
  <div class="shadowText">
    <p id="suggestionQuestion" class="activityCardFont">Förbättringarbetets resurser</p>
  </div>
  <div class="inputBox">
    <form
      action="/url"
      method="POST"
      class="inputBoxForm"
      id="inputBoxFormIdeas"
    >
      <input
        type="text"
        class="inputBoxForm"
        id="inputBoxFormResourcesInput"
        placeholder="Vilka resurser har förbättringsarbetet? Antal veckor, budget etc."
      />
    </form>
  </div>
</div>
</div>`
  );
}
