import {updateImprovementWorkMembers, 
  getHospitals,
  getUsersForImprovementWork,
  getImprovementWorkResponsible,
  getUnits,
  getObjectById,
  updateInstance,
  getTasks,
  createTaskOnImprovementWork,
  deleteInstance,
  improvementWork, 
  getImprovementWorks
} from "../Database/database.js";
import { Timestamp } from "firebase/firestore";
import {setSelectedEmployeesEmpty,
  loadEmployees,
  assignedEmployer,getSelectedEmployees,
} from "../initiateForbattringsarbete/initiateIdea.js";

export function loadPage(projectId: string) {}

export async function loadImprovementWorkTasks(
  projectId: string,
  stage: string,
  stageListClassName: string,
  taskListId: string,
  closeBtnId: string,
  taskModalId: string,
  taskTitleId: string,
  taskEndDateId: string,
  taskStartDateId: string,
  taskContentId: string,
  saveBtnId: string,
  deleteBtnId: string
) {
  //Event listener that closes task modal
  const tasks = await getTasks({ improvementWorkId: projectId, stage: stage });
  const taskListElement = $("#" + taskListId);
  modalEventListeners(
    projectId,
    stage,
    stageListClassName,
    closeBtnId,
    taskModalId,
    taskTitleId,
    taskEndDateId,
    taskStartDateId,
    taskContentId,
    taskListId,
    saveBtnId,
    deleteBtnId
  );
  if (taskListElement) {
    tasks.forEach(async (task) => {
      const taskElement = $("<li>");
      const textSpan = $("<span>").text(task.name);
      const checkbox = $("<input>").attr("type", "checkbox");
      if (task.completed) {
        checkbox.prop("checked", true);
      }
      taskElement.append(textSpan, checkbox);
      taskElement.addClass(stageListClassName);
      taskElement.on("click", function () {
        let startDateFormatted = formatDate(task.startDate);
        let endDateFormatted = formatDate(task.deadlineDate);
        openTaskModal(
          task.name,
          task.content,
          task.id,
          startDateFormatted,
          endDateFormatted,
          taskModalId,
          taskTitleId,
          taskContentId,
          taskStartDateId,
          taskEndDateId
        );
      });
      checkbox.on("click", (event) => {
        event.stopPropagation();
        if (checkbox.prop("checked")) {
          console.log("Checkbox is checked.");
          updateInstance("task", task.id, { completed: true });
        } else {
          console.log("Checkbox is unchecked.");
          updateInstance("task", task.id, { completed: false });
        }
      });

      taskListElement.append(taskElement); // Stop if the container isn't found
    });
  } else {
    console.log("Task container not found");
  }
}

//Opens modal for a task
export function openTaskModal(
  taskTitle: string,
  taskContent: string,
  taskId: string,
  taskStart: string,
  taskEnd: string,
  taskModalId: string,
  taskTitleId: string,
  taskContentId: string,
  taskStartDateId: string,
  taskEndDateId: string
) {
  const taskModal = document.getElementById(taskModalId);
  const title = document.getElementById(taskTitleId) as HTMLInputElement;
  const content = document.getElementById(taskContentId) as HTMLTextAreaElement;
  const startDate = document.getElementById(
    taskStartDateId
  ) as HTMLInputElement;
  const endDate = document.getElementById(taskEndDateId) as HTMLInputElement;
  if (taskModal) {
    taskModal.setAttribute("data-task-id", taskId);
    if (title && content && startDate && endDate) {
      title.value = "";
      content.value = "";
      startDate.value = "";
      endDate.value = "";

      if (
        taskTitle !== "" ||
        taskContent !== "" ||
        taskStart !== "" ||
        taskEnd !== ""
      ) {
        title.value = taskTitle;
        content.value = taskContent;
        startDate.value = taskStart;
        endDate.value = taskEnd;
      }
    }
    taskModal.style.display = "block";
  }
}

//Adds eventlistener to close modal
function modalEventListeners(
  projectId: string,
  stage: string,
  stageListClassName: string,
  closeBtnId: string,
  taskModalId: string,
  taskTitleId: string,
  taskEndDateId: string,
  taskStartDateId: string,
  taskContentId: string,
  taskListId: string,
  saveBtnId: string,
  deleteBtnId: string
) {
  const closeButton = document.getElementById(closeBtnId);
  const taskModal = document.getElementById(taskModalId);
  const saveButton = document.getElementById(saveBtnId);
  const deleteButton = document.getElementById(deleteBtnId);
  if (closeButton && taskModal) {
    const closeBtnListener = closeButton.hasAttribute(
      "data-has-event-listener"
    );
    if (!closeBtnListener) {
      closeButton.setAttribute("data-has-event-listener", "true");
      closeButton.addEventListener("click", () => {
        taskModal.style.display = "none"; //Closes modal
      });
    }
  }

  if (saveButton && taskModal) {
    const saveBtnListener = saveButton.hasAttribute("data-has-event-listener");
    if (!saveBtnListener) {
      saveButton.setAttribute("data-has-event-listener", "true");
      saveButton.addEventListener("click", () => {
        createNewOrSaveTask(
          projectId,
          stage,
          stageListClassName,
          taskModalId,
          taskTitleId,
          taskEndDateId,
          taskStartDateId,
          taskContentId,
          taskListId,
          closeBtnId,
          saveBtnId,
          deleteBtnId
        );
        taskModal.style.display = "none"; //Closes modal
      });
    }
  }

  if (deleteButton && taskModal) {
    const deleteBtnListener = deleteButton.hasAttribute(
      "data-has-event-listener"
    );
    if (!deleteBtnListener) {
      deleteButton.setAttribute("data-has-event-listener", "true");
      deleteButton.addEventListener("click", () => {
        deleteTask(
          projectId,
          stage,
          stageListClassName,
          taskModalId,
          taskListId,
          taskTitleId,
          taskEndDateId,
          taskStartDateId,
          taskContentId,
          saveBtnId,
          deleteBtnId,
          closeBtnId
        );
        taskModal.style.display = "none"; //Closes modal
      });
    }
  }
}

async function createNewOrSaveTask(
  projectId: string,
  stage: string,
  stageListClassName: string,
  taskModalId: string,
  taskTitleId: string,
  taskEndDateId: string,
  taskStartDateId: string,
  taskContentId: string,
  taskListId: string,
  closeBtnId: string,
  saveBtnId: string,
  deleteBtnId: string
) {
  //title: string, deadline : Date, startDate: Date, stage: string, content: string
  const modalElement = document.getElementById(taskModalId) as HTMLElement;
  const titleElement = document.getElementById(taskTitleId) as HTMLInputElement;
  const deadlineElement = document.getElementById(
    taskEndDateId
  ) as HTMLInputElement;
  const startElement = document.getElementById(
    taskStartDateId
  ) as HTMLInputElement;
  const contentElement = document.getElementById(
    taskContentId
  ) as HTMLTextAreaElement;
  const taskId = modalElement.getAttribute("data-task-id");

  if (taskId === "") {
    //Creates new task
    const improvementWork = await getObjectById("improvementWork", projectId);
    if (titleElement && contentElement && deadlineElement && startElement) {
      if (
        (titleElement.value !== "",
        deadlineElement.value !== "",
        startElement.value !== "",
        contentElement.value !== "")
      ) {
        let deadline: Date = new Date(deadlineElement.value); //Date string to timestamp
        let startDate: Date = new Date(startElement.value); //Date string to timestamp
        createTaskOnImprovementWork(
          null,
          projectId,
          titleElement.value,
          deadline,
          startDate,
          improvementWork.currentStage,
          contentElement.value
        );
        const listElement = document.getElementById(
          taskListId
        ) as HTMLUListElement;
        listElement.innerHTML = "";
        loadImprovementWorkTasks(
          projectId,
          stage,
          stageListClassName,
          taskListId,
          closeBtnId,
          taskModalId,
          taskTitleId,
          taskEndDateId,
          taskStartDateId,
          taskContentId,
          saveBtnId,
          deleteBtnId
        );
      } else {
        console.log("All input fields for the task where not filled in");
      }
    } else {
      console.log("All input element where not found");
    }
  } else {
    //Updates old task
    if (taskId) {
      let deadline: Date = new Date(deadlineElement.value); //Date string to timestamp
      let startDate: Date = new Date(startElement.value); //Date string to timestamp
      await updateInstance("task", taskId, {
        name: titleElement.value,
        deadlineDate: deadline,
        startDate: startDate,
        content: contentElement.value,
      });
      const listElement = document.getElementById(
        taskListId
      ) as HTMLUListElement;
      listElement.innerHTML = "";
      loadImprovementWorkTasks(
        projectId,
        stage,
        stageListClassName,
        taskListId,
        closeBtnId,
        taskModalId,
        taskTitleId,
        taskEndDateId,
        taskStartDateId,
        taskContentId,
        saveBtnId,
        deleteBtnId
      );
      //loadPage(projectId); //Loads tasks since updated ASK for help
    }
  }
}

async function deleteTask(
  projectId: string,
  stage: string,
  stageListClassName: string,
  taskModalId: string,
  taskListId: string,
  taskTitleId: string,
  taskEndDateId: string,
  taskStartDateId: string,
  taskContentId: string,
  saveBtnId: string,
  deleteBtnId: string,
  closeBtnId: string
) {
  const modalElement = document.getElementById(taskModalId) as HTMLElement;
  const taskId = modalElement.getAttribute("data-task-id");
  if (taskId) {
    await deleteInstance("task", taskId);
    const listElement = document.getElementById(taskListId) as HTMLUListElement;
    listElement.innerHTML = "";
    loadImprovementWorkTasks(
      projectId,
      stage,
      stageListClassName,
      taskListId,
      closeBtnId,
      taskModalId,
      taskTitleId,
      taskEndDateId,
      taskStartDateId,
      taskContentId,
      saveBtnId,
      deleteBtnId
    );
  }
}

export function formatDate(timeStamp: Timestamp): string {
  const date = timeStamp.toDate();
  const day: string = String(date.getDate()).toString();
  const month: string = String(date.getMonth() + 1).toString(); // Adding 1 because months are zero-based
  const year: string = String(date.getFullYear()).toString();

  if (parseInt(month, 10) < 10 && parseInt(day, 10) > 9) {
    return `${year}-0${month}-${day}`;
  } else if (parseInt(month, 10) > 9 && parseInt(day, 10) < 10) {
    return `${year}-${month}-0${day}`;
  } else if (parseInt(month, 10) < 10 && parseInt(day, 10) < 10) {
    return `${year}-0${month}-0${day}`;
  } else {
    return `${year}-${month}-${day}`;
  }
}
function timestamptoDate(timestamp: any): Date | null {
  if (timestamp instanceof Timestamp) {
    const date = timestamp.toDate();
    return date;
  } else {
    console.error("Invalid timestamp type.");
    return null;
  }
}

function getDaysMonthYear(startDate: Date | null): string {
  const currentTime = new Date();
  if (startDate) {
    const millisecondsDifference = currentTime.getTime() - startDate.getTime();

    // Convert milliseconds to seconds
    const secondsDifference = millisecondsDifference / 1000;

    // Calculate years, months, and days
    const years = Math.floor(secondsDifference / (365.25 * 24 * 60 * 60));
    const remainingSeconds = secondsDifference % (365.25 * 24 * 60 * 60);
    const months = Math.floor(remainingSeconds / (30 * 24 * 60 * 60)); // Assuming 30 days per month
    const days = Math.floor(
      (remainingSeconds % (30 * 24 * 60 * 60)) / (24 * 60 * 60)
    );

    return ` ${years} år, ${months} månader, ${days} dagar.`;
  } else {
    return "Det finns inget startdatum registrerat";
  }
}

export async function loadProjectHeader(improvementWork: improvementWork) {
  // Show loading indicators
  const loadingIndicators = document.querySelectorAll(".loadingIndicator");
  loadingIndicators.forEach((indicator) => {
    (indicator as HTMLParagraphElement).style.display = "block";
  });

  const titleP = document.getElementById("title") as HTMLTextAreaElement;
  titleP.textContent = improvementWork.name;

  const aAreaElement = document.getElementById("pArea") as HTMLParagraphElement;
  aAreaElement.textContent = improvementWork.area;

  const resources = document.getElementById(
    "pResources"
  ) as HTMLTextAreaElement;
  resources.textContent = improvementWork.resources;
  
  const time = document.getElementById("pTime") as HTMLTextAreaElement;
  if (improvementWork.completed) { 
      if (improvementWork.endDate instanceof Timestamp) {
          time.textContent = "Förbättringsarbetet avslutades: " + formatDate(improvementWork.endDate);
      } else { 
          console.error("Inte en Timestamp");
      }
  } else {
  let formatdateStarted = timestamptoDate(improvementWork.dateStarted); 
  time.textContent = "Tid sen startat förbättringsarbete: " + getDaysMonthYear(formatdateStarted);
  }
  
  const aUnitArray = await getUnits({ unitId: improvementWork.unitId });
        //this code loads the current members
    const projectLeader = await getImprovementWorkResponsible(improvementWork.id!);
    const members = await getUsersForImprovementWork(improvementWork.id!);
    
    const aProjectLeader = document.getElementById("pLeader")  as HTMLParagraphElement;
    aProjectLeader.textContent = projectLeader.firstName +" "+ projectLeader.lastName + " ("+projectLeader.email +")";
    
    const aMembers = document.getElementById("pMembers") as HTMLParagraphElement;
    const userNames = members.map(member => `${member.firstName} ${member.lastName}`);
    aMembers.textContent = userNames.join(', ');
        //stop loding current members
    if (aUnitArray.length === 0) {
        console.log("Error loading the units.")
    }
    
    const aUnit = aUnitArray[0];
    const aUnitElement = document.getElementById("pUnit") as HTMLParagraphElement;
    aUnitElement.textContent = aUnit.name;

  const hospitalArray = await getHospitals({ hospitalId: aUnit.hospitalId });
  if (hospitalArray.length === 0) {
    console.log("Error loading the hospitals.");
  }
  const aHospital = hospitalArray[0];
  const aHospitalElement = document.getElementById(
    "pHospital"
  ) as HTMLParagraphElement;
  aHospitalElement.textContent = aHospital.name;

  // Close loading indicators
  loadingIndicators.forEach((indicator) => {
    (indicator as HTMLParagraphElement).style.display = "none";
  });
}
function getDivWithIdFromHTML(
  path: string,
  id: string
): Promise<HTMLElement | null> {
  // Returning a Promise
  return fetch(path)
    .then((response) => response.text())
    .then((html) => {
      // Create a temporary container
      const container = document.createElement("div");
      container.innerHTML = html;

      // Get the modal content from the container
      const element = container.querySelector(id) as HTMLElement;
      return element;
    })
    .catch((error) => {
      console.error("Error loading modal content:", error);
      return null;
    });
}

export async function loadMemberModal(projectId: string) {
    const modalContent = await getDivWithIdFromHTML('../projectPage/addMemberModal.html', '#addMemberModal');
    var link = document.createElement('link');
    link.rel = 'stylesheet';
    link.type = 'text/css';
    link.href = '../initiateForbattringsarbete/initiateIdea.css'; 

    setSelectedEmployeesEmpty();

    if (modalContent) {
      $('#container-fluid').append(link);
        $('#container-fluid').append(modalContent);
        ($(modalContent) as any).modal('show');
    

        const selectEmployeeChangeSection = document.querySelector('#selectEmployee') as HTMLButtonElement;
        const upperCloseButton = modalContent.querySelector('#upperCloseMemberModal') as HTMLButtonElement;
        const lowerCloseButton = modalContent.querySelector('#lowerCloseMemberModal') as HTMLButtonElement;
        const selectEmployeeSaveButton = document.querySelector('#selectEmployeeSaveButton')as HTMLButtonElement;

        if (selectEmployeeChangeSection) {
            selectEmployeeChangeSection.addEventListener('change', assignedEmployer);
        }
        if(upperCloseButton || lowerCloseButton){
            upperCloseButton.addEventListener('click', closeModal)
            lowerCloseButton.addEventListener('click', closeModal)   
        }
        if(selectEmployeeSaveButton){
            selectEmployeeSaveButton.addEventListener('click', saveModal)
        }
        
        loadEmployees();
        setPreselectedMembers(projectId);

        function closeModal() {
            
            if (modalContent) {
                ($(modalContent) as any).modal('hide');
                $('#container-fluid').find(modalContent).remove();
                // Clear event listeners
                selectEmployeeChangeSection.removeEventListener('change', assignedEmployer);
                upperCloseButton.removeEventListener('click', closeModal);
                lowerCloseButton.removeEventListener('click', closeModal);
                selectEmployeeSaveButton.removeEventListener('click', saveModal);
            }
        }

        async function saveModal() {
            console.log("inne i save modal")
            const newMemberList = getSelectedEmployees();
            console.log(newMemberList);
            const newProjectLeaderElement = document.getElementById("projectLeader") as HTMLSelectElement;
            const newProjectLeaderId = newProjectLeaderElement.options[newProjectLeaderElement.selectedIndex].value;
            await updateImprovementWorkMembers(projectId, newProjectLeaderId,newMemberList)
            closeModal()
            //loadProjectHeader(improvementWork);

            const newProjectLeaderUser = await getImprovementWorkResponsible(projectId);
            const aProjectLeader = document.getElementById("pLeader")  as HTMLParagraphElement;
            aProjectLeader.textContent = ""
            aProjectLeader.textContent = newProjectLeaderUser.firstName +" "+ newProjectLeaderUser.lastName + " ("+newProjectLeaderUser.email +")";

            const userMemberArray = await getUsersForImprovementWork(projectId);
            const aMembers = document.getElementById("pMembers") as HTMLParagraphElement;
            aMembers.textContent =""
            const userNames = userMemberArray.map(member => `${member.firstName} ${member.lastName}`);
            aMembers.textContent = userNames.join(', ');

        }
        
    }
}


// Your async function to set preselected members
async function setPreselectedMembers(improvementWorkId: string) {
    const projectLeader = await getImprovementWorkResponsible(improvementWorkId);
    const members = await getUsersForImprovementWork(improvementWorkId);

    const selectProjectLeaderHTML = $('#projectLeader');
    const selectEmployeeHTML = $('#selectEmployee');
    
    selectProjectLeaderHTML.find('option').each(function () {
        if ($(this).val() == projectLeader.id){
            $(this).prop('selected', true);
            return false;
        }
    });
    members.forEach(member => {
        selectEmployeeHTML.find('option').each(function () {
            if ($(this).val() == member.id) {
                $(this).prop('selected', true);
                assignedEmployer();
                return false; // This will exit the inner each loop
            }
        });
    });
}

