import { loadCurrentProjects } from "../SidebarAndHeader/pageHandler.js";
import {getObjectById, finalizeImprovementWork, updateInstance, getUnits, getHospitals, getImprovementWorkResponsible, getUsersForImprovementWork, getUsers, createdDocumentInKnowledgeBank, getDocumentsByImprovementWork} from "../Database/database.js";
import { updateDoc, doc } from "firebase/firestore";
import { loadProjectPageP, loadProjectPageS,loadProjectPageG, loadProjectPageA } from "../SidebarAndHeader/pageHandler.js";
import {loadPauseRestartButton} from "../projectPage/projectPageP.js";
import {loadMemberModal, loadProjectHeader, loadImprovementWorkTasks, openTaskModal} from "../projectPage/projectPage.js";



async function updateStage(stage : string, id : string){
    updateInstance("improvementWork", id, {currentStage:stage}) 
    }

export async function loadPage (projectId : string) {
    const container = document.querySelector('#container-fluid');
    const nextPageButton = document.getElementById('toCurrentProjects') as HTMLElement;

    if (!container) return; // Stop if the container isn't found
    
    updateStage("Act", projectId);



    fetch('projectPage/projectPageA.html')
            .then(response => response.text())
            .then(data => {
                // Create a temporary container element to hold the loaded content
                const tempContainer = document.createElement('div');
                tempContainer.innerHTML = data;

                // Select the specific element with the class you want
                const specificElement = tempContainer.querySelector('.wholePageA');
            

                if (specificElement) {
                    // Insert the specific element into the target element
                    container.insertBefore(specificElement, container.firstChild);
                    
                    initializeTooltips();
                }
                
            })

            await addDocumentsToPage(projectId);

            container.addEventListener('click', async (event: Event) => {
                const target = event.target as HTMLElement; // Type assertion
                const pdfInput = document.getElementById('pdfInput') as HTMLInputElement;
                const pdfLinkContainer = document.getElementById('pdfLink');
                if(target.id === 'nolansToP'){
                    loadProjectPageP(projectId);
                } else if(target.id === 'nolansToG'){
                    loadProjectPageG(projectId);
                } else if(target.id === 'nolansToS'){
                    loadProjectPageS(projectId);
                } else if(target.id === "saveEditBtnA") {
                    saveImprovementWorkEditA(projectId);
                    
                } else if(target.id === "addMemberButton") {
                    loadMemberModal(projectId);
                }else if (target.id === "uploadButton") {
                    if((await getDocumentsByImprovementWork(projectId)).length <= 5)  {
                        pdfInput.addEventListener('change', async function () {
                        

                        
                            if (this.files && this.files[0]) {
                                const file = this.files[0];
                                const reader = new FileReader();
                                const tempUrl = URL.createObjectURL(file);
    
                                try{
                                await createdDocumentInKnowledgeBank(tempUrl, file.name, projectId);
                                await addDocumentsToPage(projectId);
    
                            } catch (error) {
                                console.error('Error uploading file and adding document to Firestore:', error);
                            }
                                reader.onload = function (e: ProgressEvent<FileReader>) {
                                    const target = e.target as FileReader;
                                    if (target.result && pdfLinkContainer) {
                                        const fileName = file.name;
                                        pdfLinkContainer.innerHTML = `<a href="${target.result}" target="_blank">${fileName}</a>`;
                                    }
                                };
                    
                                reader.readAsDataURL(file);
                            }
                        });
                    } else {
                        alert("Document limit already reached for this project")
                    }
                } else if(target.id === "new-task-button-a") {
                    openTaskModal("","", "", "", "", 'task-modal-a', 'task-title-a', 'task-content-a', 'task-start-a', 'task-end-a');
                }
        });
            
            function initializeTooltips() {
                // Use jQuery to select the elements and initialize the tooltip
                ($('[data-toggle="tooltip"]') as any).tooltip({ delay: { "show": 0, "hide": 100 } });
            }
            loadImprovementWorkDataPageA(projectId);
            loadImprovementWorkTasks(projectId, "Act", 'tasksStageA', 'tasksA', 'close-task-modal-a', 'task-modal-a', 'task-title-a', 'task-end-a', 'task-start-a', 'task-content-a', 'save-task-button-a', 'delete-task-button-a');
            
}

async function addDocumentsToPage(projectId: string) {
    const documents = await getDocumentsByImprovementWork(projectId);
    console.log(documents);

    for (let i = 0; i < documents.length; i++) {
        const pdfLinkContainer = document.getElementById(`pdfLink${i}`);
        if(pdfLinkContainer) {
            pdfLinkContainer.innerHTML = `<a href="${documents[i].url}" target="_blank">${documents[i].name}</a>`; 
        }
        
    }
}

async function loadImprovementWorkDataPageA(projectId:string) {
    const improvementWork = await getObjectById('improvementWork', projectId);
    const a1 = document.getElementById("a1") as HTMLTextAreaElement;
    const a2 = document.getElementById("a2") as HTMLTextAreaElement;
    const a3 = document.getElementById("a3") as HTMLTextAreaElement;
    const a4 = document.getElementById("a4") as HTMLTextAreaElement;
    a1.value = improvementWork.a1;
    a2.value = improvementWork.a2;
    a3.value = improvementWork.a3;
    a4.value = improvementWork.a4;

    //Checks if user is manager
    var user = localStorage.getItem('user');
    if(user){
        var isManager = JSON.parse(user)[0].manager;
        if(isManager) {
            loadPauseRestartButton(projectId, improvementWork.ongoing, "left-content-div-a"); //Loads pause/restart button into html
        }
    }
    loadProjectHeader(improvementWork);
    loadInitiateAndFinalizeBtn(projectId, improvementWork.completed);
}


async function finalize(projectId : string) {
    var user = localStorage.getItem('user');
    if(user){
        var userId = JSON.parse(user)[0].id;
    }
    if (await finalizeImprovementWork(userId, projectId)) {
        loadCurrentProjects();
        return true
    } else {
        return false;
    }
}


async function saveImprovementWorkEditA(projectId:string) {
    const a1 = document.getElementById("a1") as HTMLTextAreaElement;
    const a2 = document.getElementById("a2") as HTMLTextAreaElement;
    const a3 = document.getElementById("a3") as HTMLTextAreaElement;
    const a4 = document.getElementById("a4") as HTMLTextAreaElement;

    let update = { a1: a1.value, a2: a2.value, a3: a3.value, a4: a4.value };
    updateInstance("improvementWork", projectId, update);
    return true;
}

async function checkInputFields(projectId: string, formId: string) {
    const form = document.getElementById(formId) as HTMLFormElement;
        if (form.checkValidity()) {
            (saveImprovementWorkEditA(projectId)) //Returns true if successful
                .then(async (success: boolean) => {
                    if (success === true) {
                        if (await finalize(projectId)) {
                            updateStage("Finished", projectId);
                        } else {
                            // TODO nicer error message
                            alert("Du kan inte avsluta detta förbättringsarbetet")
                        };

                    } 
                })
        } else {
            alert('Please fill in all required fields.');
        }
}

function loadInitiateAndFinalizeBtn(projectId : string, completed : boolean) {
    if(!completed) {
        const nolanModelElement = document.getElementById("projectPageNolandsModel") as HTMLElement;

        //Creates buttons with classname, id, attribute and text
        const finalizeButton = document.createElement("button");
        const iterationButton = document.createElement("button");
        if (nolanModelElement) {
            finalizeButton.id = "toCurrentProjects";
            iterationButton.id = "toCurrentProjects";
            finalizeButton.classList.add("btn", "btn-secondary");
            iterationButton.classList.add("btn", "btn-secondary");
            iterationButton.setAttribute('role', 'button');
            finalizeButton.setAttribute('role', 'button');
            finalizeButton.textContent = 'Avsluta Förbättringsarbete';
            iterationButton.textContent = 'En till iteration';
        }
        //Append buttons to html
        nolanModelElement.append(iterationButton);
        nolanModelElement.append(finalizeButton);

        //Eventlistener for button
        finalizeButton.addEventListener('click', function() {
            checkInputFields(projectId, "formPageA");;
        });
    }
}


