import { loadProjectPageP, loadProjectPageS } from "../SidebarAndHeader/pageHandler.js";
import {getObjectById, updateInstance, getHospitals, getUnits, getImprovementWorkResponsible, getUsersForImprovementWork, getUsers, createdDocumentInKnowledgeBank, getDocumentsByImprovementWork } from "../Database/database.js";
import {loadPauseRestartButton} from "../projectPage/projectPageP.js";
import {loadMemberModal, loadProjectHeader, loadImprovementWorkTasks, openTaskModal} from "../projectPage/projectPage.js";

async function updateStage(stage : string, id : string){
    updateInstance("improvementWork", id, {currentStage:stage}) 
}

export async function loadPage (projectId : string) {
    const container = document.querySelector('#container-fluid');
    const nextPageButton = document.getElementById('toS') as HTMLElement;

    if (!container) return; // Stop if the container isn't found
    
        updateStage("Do", projectId);
        
    

    fetch('projectPage/projectPageG.html')
            .then(response => response.text())
            .then(data => {
                // Create a temporary container element to hold the loaded content
                const tempContainer = document.createElement('div');
                tempContainer.innerHTML = data;

                // Select the specific element with the class you want
                const specificElement = tempContainer.querySelector('.wholePageG');
            

                if (specificElement) {
                    // Insert the specific element into the target element
                    container.insertBefore(specificElement, container.firstChild);
                    
                    initializeTooltips();
                }
                
            })
            .catch(error => {
                console.error('Failed to load the external HTML file:', error);
            });
            const documents = await getDocumentsByImprovementWork(projectId);
            console.log(documents);
            const pdfLinkContainer = document.getElementById('pdfLink');
            if(pdfLinkContainer) {
                documents.forEach(async (document) => {
                    pdfLinkContainer.innerHTML = `<a href="${document.url}" target="_blank">${document.name}</a>`;
                } );
                
            }

            await addDocumentsToPage(projectId);

            container.addEventListener('click', async (event: Event) => {
                const target = event.target as HTMLElement; // Type assertion
                const pdfInput = document.getElementById('pdfInput') as HTMLInputElement;
                const pdfLinkContainer = document.getElementById('pdfLink');
                if (target.id === 'toS') {
                    checkInputFields(projectId, 'formPageG');
                }else if(target.id === 'nolansToP'){
                    loadProjectPageP(projectId);
                } else if(target.id === "addMemberButton") {
                    loadMemberModal(projectId);
                } else if(target.id === "saveEditBtnG") {
                    saveImprovementWorkEditG(projectId);
                }  else if (target.id === "uploadButton") {
                    if((await getDocumentsByImprovementWork(projectId)).length <= 5)  {
                        pdfInput.addEventListener('change', async function () {
                        

                        
                            if (this.files && this.files[0]) {
                                const file = this.files[0];
                                const reader = new FileReader();
                                const tempUrl = URL.createObjectURL(file);
    
                                try{
                                await createdDocumentInKnowledgeBank(tempUrl, file.name, projectId);
                                await addDocumentsToPage(projectId);
    
                            } catch (error) {
                                console.error('Error uploading file and adding document to Firestore:', error);
                            }
                                reader.onload = function (e: ProgressEvent<FileReader>) {
                                    const target = e.target as FileReader;
                                    if (target.result && pdfLinkContainer) {
                                        const fileName = file.name;
                                        pdfLinkContainer.innerHTML = `<a href="${target.result}" target="_blank">${fileName}</a>`;
                                    }
                                };
                    
                                reader.readAsDataURL(file);
                            }
                        });
                    } else {
                        alert("Document limit already reached for this project")
                    }
                }
                if (target.id === "new-task-button-g") {
                    openTaskModal("","", "", "", "", 'task-modal-g', 'task-title-g', 'task-content-g', 'task-start-g', 'task-end-g');
                }
            });
            loadImprovementWorkDataPageG(projectId);
            loadImprovementWorkTasks(projectId, "Do", 'tasksStageG', 'tasksG', 'close-task-modal-g', 'task-modal-g', 'task-title-g', 'task-end-g', 'task-start-g', 'task-content-g', 'save-task-button-g', 'delete-task-button-g');
            function initializeTooltips() {
                // Use jQuery to select the elements and initialize the tooltip
                ($('[data-toggle="tooltip"]') as any).tooltip({ delay: { "show": 0, "hide": 100 } });
            }
}

async function addDocumentsToPage(projectId: string) {
    const documents = await getDocumentsByImprovementWork(projectId);
    console.log(documents);

    for (let i = 0; i < documents.length; i++) {
        const pdfLinkContainer = document.getElementById(`pdfLink${i}`);
        if(pdfLinkContainer) {
            pdfLinkContainer.innerHTML = `<a href="${documents[i].url}" target="_blank">${documents[i].name}</a>`; 
        }
        
    }
}


async function loadImprovementWorkDataPageG(projectId:string) {
    const improvementWork = await getObjectById('improvementWork', projectId);
    const g1 = document.getElementById("g1") as HTMLTextAreaElement;
    const g2 = document.getElementById("g2") as HTMLTextAreaElement;
    const g3 = document.getElementById("g3") as HTMLTextAreaElement;
    const g4 = document.getElementById("g4") as HTMLTextAreaElement;
    const g5 = document.getElementById("g5") as HTMLTextAreaElement;
    g1.value = improvementWork.g1;
    g2.value = improvementWork.g2;
    g3.value = improvementWork.g3;
    g4.value = improvementWork.g4;
    g5.value = improvementWork.g5;

    //Checks if user is manager
    var user = localStorage.getItem('user');
    if(user){
        var isManager = JSON.parse(user)[0].manager;
        if(isManager) {
            loadPauseRestartButton(projectId, improvementWork.ongoing, "left-content-div-g"); //Loads pause/restart button into html
        }
    }
    loadProjectHeader(improvementWork);
}

async function saveImprovementWorkEditG(projectId:string) {
    const g1 = document.getElementById("g1") as HTMLTextAreaElement;
    const g2 = document.getElementById("g2") as HTMLTextAreaElement;
    const g3 = document.getElementById("g3") as HTMLTextAreaElement;
    const g4 = document.getElementById("g4") as HTMLTextAreaElement;
    const g5 = document.getElementById("g5") as HTMLTextAreaElement;

    let update = { g1: g1.value, g2: g2.value, g3: g3.value, g4: g4.value, g5: g5.value };
    updateInstance("improvementWork", projectId, update);
    return true;
}


function checkInputFields(projectId: string, formId: string) {
    const form = document.getElementById(formId) as HTMLFormElement;
        if (form.checkValidity()) {
            (saveImprovementWorkEditG(projectId)) //Returns true if successful
                .then((success: boolean) => {
                    if (success === true) {
                        loadProjectPageS(projectId); //Loads next page if save was successful
                    } 
                })
        } else {
            alert('Please fill in all required fields.');
        }
}