import { loadProjectPageG } from "../SidebarAndHeader/pageHandler.js";
import {getObjectById, updateInstance, getTasks, getHospitals, getUnits, startOrPauseImprovementWork, getImprovementWorkResponsible, getUsersForImprovementWork, getUsers, createTaskOnImprovementWork, deleteInstance} from "../Database/database.js";
import { close } from "fs";
import {loadMemberModal, loadImprovementWorkTasks, openTaskModal,loadProjectHeader} from "../projectPage/projectPage.js";

async function updateStage(stage : string, id : string) {
    updateInstance("improvementWork", id, {currentStage:stage}) 
}


export function loadPage (projectId:string) {

    const container = document.querySelector('#container-fluid');
    if (!container) return; // Stop if the container isn't found
    updateStage("Plan", projectId);

    
    fetch('projectPage/projectPageP.html')
            .then(response => response.text())
            .then(data => {
                // Create a temporary container element to hold the loaded content
                const tempContainer = document.createElement('div');
                tempContainer.innerHTML = data;
                // Select the specific element with the class you want
                const specificElement = tempContainer.querySelector('.wholePageP');
                if (specificElement) {
                    // Insert the specific element into the target element
                    container.insertBefore(specificElement, container.firstChild);
                    initializeTooltips();
                }
            })
            .catch(error => {
                console.error('Failed to load the external HTML file:', error);
            });
        
container.addEventListener('click', (event: Event) => {
    const target = event.target as HTMLElement; // Type assertion

    if (target.matches('[data-toggle="tooltip"]') && !target.dataset.tooltipInitialized) {
        ($(target) as any).tooltip();
        target.dataset.tooltipInitialized = 'true';
    }
    
    if (target.id === 'toG') {
        checkInputFields(projectId, 'formPageP');
    }

    if(target.id === "saveEditBtnP") {
        saveImprovementWorkEditP(projectId);
    }
    
    if(target.id === "new-task-button-p") {
        openTaskModal("","", "", "", "", 'task-modal-p', 'task-title-p', 'task-content-p', 'task-start-p', 'task-end-p');
    }
    if(target.id === "addMemberButton") {
        loadMemberModal(projectId);
    }

});
loadImprovementWorkDataPageP(projectId);
loadImprovementWorkTasks(projectId, "Plan", 'tasksStageP', 'tasksP', 'close-task-modal-p', 'task-modal-p', 'task-title-p', 'task-end-p', 'task-start-p', 'task-content-p', 'save-task-button-p', 'delete-task-button-p');
}

function initializeTooltips() {
    // Use jQuery to select the elements and initialize the tooltip
    ($('[data-toggle="tooltip"]') as any).tooltip({ delay: { "show": 0, "hide": 100 } });
}

async function loadImprovementWorkDataPageP(projectId:string) {
    const improvementWork = await getObjectById('improvementWork', projectId);
    //const titleP = document.getElementById("title") as HTMLTextAreaElement;
    const p1 = document.getElementById("p1") as HTMLTextAreaElement;
    const p2 = document.getElementById("p2") as HTMLTextAreaElement;
    const p3 = document.getElementById("p3") as HTMLTextAreaElement;
    const p4 = document.getElementById("p4") as HTMLTextAreaElement;
    const p5 = document.getElementById("p5") as HTMLTextAreaElement;
    const p6 = document.getElementById("p6") as HTMLTextAreaElement;
    const p7 = document.getElementById("p7") as HTMLTextAreaElement;
    //titleP.textContent = improvementWork.name;
    p1.value = improvementWork.p1;
    p2.value = improvementWork.p2;
    p3.value = improvementWork.p3;
    p4.value = improvementWork.p4;
    p5.value = improvementWork.p5;
    p6.value = improvementWork.p6;
    p7.value = improvementWork.p7;
    
    //Checks if user is manager
    var user = localStorage.getItem('user');
    if(user){
        var isManager = JSON.parse(user)[0].manager;
        if(isManager) {
            loadPauseRestartButton(projectId, improvementWork.ongoing, "left-content-div-p"); //Loads pause/restart button into html
        }
    }
    loadProjectHeader(improvementWork);
}

async function saveImprovementWorkEditP(projectId:string) {
    const p1 = document.getElementById("p1") as HTMLTextAreaElement;
    const p2 = document.getElementById("p2") as HTMLTextAreaElement;
    const p3 = document.getElementById("p3") as HTMLTextAreaElement;
    const p4 = document.getElementById("p4") as HTMLTextAreaElement;
    const p5 = document.getElementById("p5") as HTMLTextAreaElement;
    const p6 = document.getElementById("p6") as HTMLTextAreaElement;
    const p7 = document.getElementById("p7") as HTMLTextAreaElement;

    let update = { p1: p1.value, p2: p2.value, p3: p3.value, p4: p4.value, p5: p5.value, p6: p6.value, p7: p7.value };
    updateInstance("improvementWork", projectId, update);
    return true;
}


/*Loads restart/pause button for an improvement work
Takes improvement work id and boolean if improvement work is ongoing as in parameters
*/
export function loadPauseRestartButton(projectId : string, ongoing : boolean, divId : string) {    
    
    const leftAreaElement = document.getElementById(divId) as HTMLElement;

    //Creates button with classname and text
    const pauseRestartButton = document.createElement("button");
    pauseRestartButton.id = 'startPauseButton'
    if (ongoing) {
        pauseRestartButton.textContent = "Pausa förbättringsarbete";
        pauseRestartButton.className = "pause-button"
    } else if (!ongoing) {
        pauseRestartButton.textContent = "Återuppta förbättringsarbete";
        pauseRestartButton.className = "restart-button"
    }
    //Append button to html
    leftAreaElement.append(pauseRestartButton);

    //Eventlistener for button
    pauseRestartButton.addEventListener('click', function() {
        var user = localStorage.getItem('user');
        if(user){
            var userId = JSON.parse(user)[0].id;
        }
        
        //Sets ongoing to false if true and vice versa
        (startOrPauseImprovementWork(userId, projectId)) //Returns true if successful
         .then((success: boolean) => {
            if (success === true) {
                if(pauseRestartButton.className === "restart-button") { //Changes button text to "Pausa förbättringsarbete"
                    pauseRestartButton.className = "pause-button";
                    pauseRestartButton.textContent = "Pausa förbättringsarbete";
                } else if (pauseRestartButton.className === "pause-button") { //Changes button text to "Återuppta förbättringsarbete"
                    pauseRestartButton.className = "restart-button";
                    pauseRestartButton.textContent = "Återuppta förbättringsarbete";
                }
            } 
          })
    })
}

function checkInputFields(projectId: string, formId: string) {
    const form = document.getElementById(formId) as HTMLFormElement;
        if (form.checkValidity()) {
            (saveImprovementWorkEditP(projectId)) //Returns true if successful
                .then((success: boolean) => {
                    if (success === true) {
                        loadProjectPageG(projectId); //Loads next page if save was successful
                    } 
                })
        } else {
            alert('Please fill in all required fields.');
        }
}