import { loadProjectPageA,loadProjectPageP,loadProjectPageG, loadProjectPageS } from "../SidebarAndHeader/pageHandler.js";
import {getObjectById, updateInstance, getHospitals, getUnits, getImprovementWorkResponsible, getUsersForImprovementWork, getUsers, createdDocumentInKnowledgeBank, getDocumentsByImprovementWork } from "../Database/database.js";
import {loadPauseRestartButton} from "../projectPage/projectPageP.js";
import {loadMemberModal, loadProjectHeader, loadImprovementWorkTasks, openTaskModal} from "../projectPage/projectPage.js";


async function updateStage(stage : string, id : string){
    updateInstance("improvementWork", id, {currentStage:stage}) 
    }

export async function loadPage (projectId : string) {
    const container = document.querySelector('#container-fluid');
    const nextPageButton = document.getElementById('toA') as HTMLElement;

    if (!container) return; // Stop if the container isn't found
    
        updateStage("Study", projectId);
    

    fetch('projectPage/projectPageS.html')
            .then(response => response.text())
            .then(data => {
                // Create a temporary container element to hold the loaded content
                const tempContainer = document.createElement('div');
                tempContainer.innerHTML = data;

                // Select the specific element with the class you want
                const specificElement = tempContainer.querySelector('.wholePageS');
            

                if (specificElement) {
                    // Insert the specific element into the target element
                    container.insertBefore(specificElement, container.firstChild);
                    
                    initializeTooltips();
                }
                
            })
            const documents = await getDocumentsByImprovementWork(projectId);
            console.log(documents);
            const pdfLinkContainer = document.getElementById('pdfLink');
            if(pdfLinkContainer) {
                documents.forEach(async (document) => {
                    pdfLinkContainer.innerHTML = `<a href="${document.url}" target="_blank">${document.name}</a>`;
                } );
                
            }

            await addDocumentsToPage(projectId);

            container.addEventListener('click', async (event: Event) => {
                const target = event.target as HTMLElement; // Type assertion
                const pdfInput = document.getElementById('pdfInput') as HTMLInputElement;
                const pdfLinkContainer = document.getElementById('pdfLink');
            
                if (target.id === 'toA') {
                    checkInputFields(projectId, "formPageS")
                }else if(target.id === 'nolansToP'){
                    loadProjectPageP(projectId);
                }else if(target.id === 'nolansToG'){
                    loadProjectPageG(projectId);
                } else if(target.id === "saveEditBtnS") {
                    saveImprovementWorkEditS(projectId);
                }  else if(target.id === "addMemberButton") {
                    loadMemberModal(projectId);
                }else if (target.id === "uploadButton") {
                    if((await getDocumentsByImprovementWork(projectId)).length <= 5)  {
                        pdfInput.addEventListener('change', async function () {
                        

                        
                            if (this.files && this.files[0]) {
                                const file = this.files[0];
                                const reader = new FileReader();
                                const tempUrl = URL.createObjectURL(file);
    
                                try{
                                await createdDocumentInKnowledgeBank(tempUrl, file.name, projectId);
                                await addDocumentsToPage(projectId);
    
                            } catch (error) {
                                console.error('Error uploading file and adding document to Firestore:', error);
                            }
                                reader.onload = function (e: ProgressEvent<FileReader>) {
                                    const target = e.target as FileReader;
                                    if (target.result && pdfLinkContainer) {
                                        const fileName = file.name;
                                        pdfLinkContainer.innerHTML = `<a href="${target.result}" target="_blank">${fileName}</a>`;
                                    }
                                };
                    
                                reader.readAsDataURL(file);
                            }
                        });
                    } else {
                        alert("Document limit already reached for this project")
                    }
                } else if(target.id === "new-task-button-s") {
                    openTaskModal("","", "", "", "", 'task-modal-s', 'task-title-s', 'task-content-s', 'task-start-s', 'task-end-s');
                }
            });            
            loadImprovementWorkDataPageS(projectId);
            loadImprovementWorkTasks(projectId, "Study", 'tasksStageS', 'tasksS', 'close-task-modal-s', 'task-modal-s', 'task-title-s', 'task-end-s', 'task-start-s', 'task-content-s', 'save-task-button-s', 'delete-task-button-s');
            function initializeTooltips() {
                // Use jQuery to select the elements and initialize the tooltip
                ($('[data-toggle="tooltip"]') as any).tooltip({ delay: { "show": 0, "hide": 100 } });
            }
}

async function addDocumentsToPage(projectId: string) {
    const documents = await getDocumentsByImprovementWork(projectId);
    console.log(documents);

    for (let i = 0; i < documents.length; i++) {
        const pdfLinkContainer = document.getElementById(`pdfLink${i}`);
        if(pdfLinkContainer) {
            pdfLinkContainer.innerHTML = `<a href="${documents[i].url}" target="_blank">${documents[i].name}</a>`; 
        }
        
    }
}


async function loadImprovementWorkDataPageS(projectId:string) {
    const improvementWork = await getObjectById('improvementWork', projectId);
    const s1 = document.getElementById("s1") as HTMLTextAreaElement;
    const s2 = document.getElementById("s2") as HTMLTextAreaElement;
    const s3 = document.getElementById("s3") as HTMLTextAreaElement;
    s1.value = improvementWork.s1;
    s2.value = improvementWork.s2;
    s3.value = improvementWork.s3;

    //Checks if user is manager
    var user = localStorage.getItem('user');
    if(user){
        var isManager = JSON.parse(user)[0].manager;
        if(isManager) {
            loadPauseRestartButton(projectId, improvementWork.ongoing, "left-content-div-s"); //Loads pause/restart button into html
        }
    }
    loadProjectHeader(improvementWork);
}

async function saveImprovementWorkEditS(projectId:string) {
    const s1 = document.getElementById("s1") as HTMLTextAreaElement;
    const s2 = document.getElementById("s2") as HTMLTextAreaElement;
    const s3 = document.getElementById("s3") as HTMLTextAreaElement;

    let update = { s1: s1.value, s2: s2.value, s3: s3.value };
    updateInstance("improvementWork", projectId, update);
    return true;
}

function checkInputFields(projectId: string, formId: string) {
    const form = document.getElementById(formId) as HTMLFormElement;
        if (form.checkValidity()) {
            (saveImprovementWorkEditS(projectId)) //Returns true if successful
                .then((success: boolean) => {
                    if (success === true) {
                        loadProjectPageA(projectId); //Loads next page if save was successful
                    } 
                })
        } else {
            alert('Please fill in all required fields.');
        }
}